/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.biz.api.analyse;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FastByteArrayOutputStream;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.crypto.SecureUtil;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.dal.mysql.auto.paging.PlanListPage;
import com.abssqr.plat.common.facade.constants.AbssqrConstants;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.model.analyse.PlanFundFile;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.ReportField;
import com.abssqr.plat.common.facade.model.report.wk.WkAstReportFields;
import com.abssqr.plat.common.facade.param.analyse.AnalyseBaseParam;
import com.abssqr.plat.common.facade.param.analyse.CalcFundParam;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQprParam;
import com.abssqr.plat.common.facade.param.analyse.FundPlanFileQryParam;
import com.abssqr.plat.common.facade.param.plan.PlanPageQryParam;
import com.abssqr.plat.common.facade.result.BasePage;
import com.abssqr.plat.common.facade.result.analyse.AnalysePlanItemVO;
import com.abssqr.plat.common.facade.result.analyse.CalcFundDefaultParam;
import com.abssqr.plat.common.facade.result.analyse.CalcFundResultVO;
import com.abssqr.plat.common.facade.result.analyse.PlanAnalyseVO;
import com.abssqr.plat.common.facade.service.analyse.AnalyseFundSpvService;
import com.abssqr.plat.common.facade.utils.CalcUtil;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.integration.wk.domain.config.WkSftpConfig;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.analyse.PlanFundFileRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.utils.FileType;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.report.AnalyseSpvReportManager;
import com.abssqr.plat.core.service.util.AnalyseReportUtil;
import com.general.system.common.model.Money;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.facade.current.IdentityUtil;
import com.general.system.tool.util.ToolUtil;
import com.google.common.collect.Lists;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 项目分析资金计划 管理
 * @author hanfei
 * @version $Id: AnalyseFundSpvServiceImpl.java, v 0.1 2019-09-19 4:23 PM hanfei Exp $
 */
@Service
public class AnalyseFundSpvServiceImpl extends ServiceSupport implements AnalyseFundSpvService {
    private final Logger logger = LoggerFactory.getLogger(getClass());
    private static  final List<ReportField> amtFields = Lists.newArrayList(
        WkAstReportFields.WK017,
        WkAstReportFields.WK018,
        WkAstReportFields.WK019,
        WkAstReportFields.WK001
    );
    // 资产 本金 利息 罚息
    private static  final  List<ReportField> astScale = Lists.newArrayList(
        WkAstReportFields.WK017,
        WkAstReportFields.WK018,
        WkAstReportFields.WK019);
    @Autowired
    private PlanManager             planManager;
    @Autowired
    private PlanFundFileRepo        planFundFileRepo;
    @Autowired
    private PlanAllocRepo           planAllocRepo;
    @Autowired
    private AnalyseSpvReportManager analyseSpvReportManager;
    @Autowired
    private WkSftpConfig            wkSftpConfig;
    @Autowired
    private OrgAcctDayComponent     orgAcctDayComponent;
    @Autowired
    private PlanReportRepo          planReportRepo;
    /**
     * 获取资金计划文件
     * @param param
     * @return
     */
    @Override
    public PlanFundFile getFundPlanFile(FundPlanFileQryParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotNull(param.getFileId(), "文件变号不能为空");
            PlanFundFile file = planFundFileRepo.getByFileId(param.getFileId());
            // 根据文件路径 获取文件
            FastByteArrayOutputStream stream = wkSftpConfig.downloadSftp(file.getFilePath());
            if (stream == null) {
                throw new AbssqrBizException("未找到文件", CommonErrorCodeEnum.SFTP_ERROR);
            }
            file.setFileOutStream(stream);
            return file;
        });
    }
    
    /**
     * 获取资金计划文件页面数据
     * @return
     */
    @Override
    public BasePage queryFundFile(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            // 查询计划
            analyseSpvReportManager.checkGetPlan(param.getPlanNo());
            PlanFundFile file = planFundFileRepo.queryFileInfo(param.getPlanNo());
            BasePage page = new BasePage();
            page.setDatas(Lists.newArrayList(file));
            page.setTotal(1);
            return page;
        });
    }
    
    /**
     * 上传资金计划文件
     * @param param
     * @return
     */
    @Override
    public String uploadPlanFile(FundPlanFileQprParam param) {
        return super.doOpr(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            VarChecker.checkNotNull(param.getFile(), "上传文件不能为空");
            // 查询计划
            PlanBase planBase = analyseSpvReportManager.checkGetPlan(param.getPlanNo());
            // 组装文件信息
            PlanFundFile fundFile = new PlanFundFile();
            fundFile.setFileName(param.getFile().getOriginalFilename());
            fundFile.setFileId(SequenceUtil.genFakeSeqNo());
            String suffix = FileType.getSuffixByFilename(fundFile.getFileName());
            // 随机文件名
            String pathFileName = fundFile.getFileId().concat(suffix);
            fundFile.setFilePath(wkSftpConfig.getPath().concat(File.separator).concat(pathFileName));
            fundFile.setUploadDate(SystemDateUtil.getSystemDate());
            fundFile.setUploadType(wkSftpConfig.getType());
            fundFile.setPlanNo(planBase.getPlanNo());
            try {
                String fileMd5 = SecureUtil.md5(param.getFile().getInputStream()).toUpperCase();
                fundFile.setFileMd5(fileMd5);

                // 处理上传文件
                wkSftpConfig.uploadSftp(param.getFile().getInputStream(), pathFileName, wkSftpConfig.getPath());
            } catch (IOException e) {
                throw new AbssqrBizException("处理上传文件时异常", e);
            }
            // 保存数据
            planFundFileRepo.create(fundFile);
            return fundFile.getFileId();
        });
    }
    
    /**
     * 短期资金需求计算器 默认信息
     * @param param
     * @return
     */
    @Override
    public CalcFundDefaultParam fundCalcView(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            String planNo = param.getPlanNo();
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 查询计划
            PlanBase planBase = analyseSpvReportManager.checkGetFullPlan(planNo);
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(param.getPlanNo(), ReportTypeEnum.RPT001);
            // D-1日
            Date lastDate = AnalyseReportUtil.getStatDate(trfDate, null, planReport);
            // D-30日
            final Date last30Date = DateUtil.offsetDay(lastDate, AbssqrConstants.AVG_SPARE);

            // 统计 闲置资金余额
            Map<String, PlanReportItem> itemBalMap = analyseSpvReportManager.querySpvStatPlanDayItem(
                    lastDate, planBase.getPlanNo(), CollectionUtil.newArrayList(WkAstReportFields.WK001), true);
            VarChecker.checkNotNull(itemBalMap, "[{0}-{1}]计划最近报表[{2}]未生成！", planBase.getPlanNo(), planBase.getPlanName(), DateTimeUtil.formatY_M_D(lastDate));
            // 资金余额
            Money ableBal = AnalyseReportUtil.getItemMoney(itemBalMap, WkAstReportFields.WK001);
            // 计算时间间隔，可能没有够时长的报表数据，last30Date 会根据数据日期进行变化
            AtomicInteger spare = new AtomicInteger(30);
            // 统计30日放款额
            Map<String, PlanReportItem> itemGrantMap = analyseSpvReportManager.querySpvStatPlanSectionItem(
                    last30Date, lastDate, planBase.getPlanNo(), CollectionUtil.newArrayList(WkAstReportFields.WK111), false, spare);

            // 日均放款额
            Money grantAmt = new Money();
            if (spare.get() > 0) {
                // 放款金额 spare 天平均
                grantAmt = AnalyseReportUtil.getItemMoney(itemGrantMap, WkAstReportFields.WK111)
                        .divide(new BigDecimal(spare.get()), AbssqrConstants.DECIMAL_ROUNDING_MODE);
            }
            CalcFundDefaultParam defaultParam = new CalcFundDefaultParam();
            defaultParam.setPlanNo(planNo);
            // 回款率 D-30日
            defaultParam.setRepayRate(analyseSpvReportManager.getRepayRate(last30Date, lastDate, planNo));
            // 可用余额
            defaultParam.setAbleBal(ableBal);
            // 放款金额
            defaultParam.setGrantAmt(grantAmt);
            return defaultParam;
        });
    }
    
    /**
     * 短期资金需求计算器
     * @param param
     * @return
     */
    @Override
    public CalcFundResultVO fundCalc(@Valid CalcFundParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotNull(param.getPlanNo(), "项目计划编号不能为空");
            VarChecker.checkNotNull(param.getAccountDate(), "募集日需不能为空");
            VarChecker.checkNotNull(param.getGrantAmtDesc(), "日均放款额不能为空");
            VarChecker.checkNotNull(param.getAbleBal(), "可用余额不能为空");
            VarChecker.checkNotNull(param.getRepayRate(), "回款率不能为空");
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 募集日
            Date raiseDate = DateUtil.parseDate(param.getAccountDate());
            int spare = Long.valueOf(DateUtil.betweenDay(trfDate.getStandardDate(), raiseDate, true) + 1).intValue();
            VarChecker.checkArgument(spare > 0, "募集日需要大于当前会计日");

            // 查询校验计划
            analyseSpvReportManager.checkGetPlan(param.getPlanNo());
            // 获取 期间兑付金额
    //        Money allocAmt = planAllocRepo.queryStatAllocAmtByPlanNoAndPayDate(calcDate, raiseDate, param.getPlanNo());
            // 总资产金额 查询当前资产情况
    //        Money astScale = loanInstalRepository.getPlanAstScale(param.getPlanNo());
    //        astScale = Money.createWithCent(100000000000L);
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(param.getPlanNo(), ReportTypeEnum.RPT001);
            // 统计总贷款本金余额
            Map<String, PlanReportItem> itemGrantMap = analyseSpvReportManager.querySpvStatPlanDayItem(
                    AnalyseReportUtil.getStatDate(trfDate,null, planReport), param.getPlanNo(),
                    CollectionUtil.newArrayList(WkAstReportFields.WK017), true);

            Money grantAmt = new Money(param.getGrantAmtDesc().multiply(new BigDecimal(10000)));
            // 当前资金余额 + 募集资金金额
            Money bal = new Money(param.getAbleBal().multiply(new BigDecimal(10000)))
                                .addTo(new Money(param.getScaleAmtDesc().multiply(new BigDecimal(10000))));
            BigDecimal repayRate = CalcUtil.calcRealRate(param.getRepayRate());
            // 当前资产还款总计(当前资产 * 日还款率 * 天数)
            Money scaleFlowIn = AnalyseReportUtil.getItemMoney(itemGrantMap, WkAstReportFields.WK017)
                                        .multiply(repayRate).multiply(spare);


            //        Money grantFlowIn = calcRepayArray(new Money(param.getGrantAmtDesc()), repayRate, result.getSpareDays());
            // 未来资产还款累计连加和(∑天数.日均放款*日还款率)
            double radix = NumberUtil.div((spare+1) * spare, 2);
            Money grantFlowIn = grantAmt.multiply(repayRate).multiply(radix);
            // 预计流出金额 = 日均放款预计 * 距离募集剩余天数 + 期间兑付额
            Money flowOut = grantAmt.multiply(spare);

            CalcFundResultVO result = new CalcFundResultVO();
            result.setPlanNo(param.getPlanNo());
            // 剩余天数
            result.setSpareDays(spare);
            // 预计还款流入金额 = 当前资产还款总计(当前资产 * 日还款率 * 天数) + 未来资产还款累计连加和(∑天数.日均放款*日还款率)
            result.setFlowInAmt(scaleFlowIn.add(grantFlowIn));
            // 预计募集日账户余额 = 当前资金余额 + 募集资金金额 + 预计期间还款流入 - 预计流出金额(合计放款 + 期间兑付额)
            result.setPlanAbleBal(bal.addTo(result.getFlowInAmt()).subtractFrom(flowOut));
            return result;
        });
    }
    
    /**
     * 查询项目计划分析列表
     * @return
     */
    @Override
    public BasePage queryAnalyseSpvPlan(PlanPageQryParam param) {
        return super.doQry(param, () -> {
            BasePage page = new BasePage();
            param.setPlanType(PlanTypeEnum.SPV.getCode());
            PlanListPage planList = planManager.getPlanList(param,
                    CollectionUtil.newArrayList(PlanStatusEnum.ONGOING.getCode(), PlanStatusEnum.COMPLETED.getCode()), IdentityUtil.getUserCode());
            if (ToolUtil.isEmpty(planList.getDatas())) {
                return page;
            }

            // 闲置资金余额
            List<ReportField> ableBal = Lists.newArrayList(WkAstReportFields.WK001);

            List<AnalysePlanItemVO> items = new ArrayList<>();
            planList.getDatas().forEach(one -> {
                PlanBase planBase = planManager.getPlanFullByNo(one.getPlanNo());

                AnalysePlanItemVO item = new AnalysePlanItemVO();
                item.setPlanNo(planBase.getPlanNo());
                item.setPlanName(planBase.getPlanName());

                PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
                if(planReport!=null) {
                    Date lastDate =  planReport.getRptDate().getAccountDate();
                    // D-30日
                    final Date last30Date = DateUtil.offsetDay(lastDate, AbssqrConstants.AVG_SPARE);

                    // 查询资金计划文件
                    PlanFundFile file = planFundFileRepo.queryFileInfo(planBase.getPlanNo());
                    // 查询计划对应统计项数据
                    Map<String, PlanReportItem> itemMap =
                        analyseSpvReportManager.querySpvStatPlanDayItem(lastDate, planBase.getPlanNo(), amtFields, true);

                    // 本金规模
                    item.setPrinScale(AnalyseReportUtil.getItemMoney(itemMap, WkAstReportFields.WK017));
                    // 资产规模
                    item.setAstScale(AnalyseReportUtil.getItemMoney(itemMap, astScale));
                    // 可用余额
                    item.setAbleBal(AnalyseReportUtil.getItemMoney(itemMap, ableBal));
                    // 获取 逾期率 闲置率 回收率 早偿率
                    item.setOvdRate(analyseSpvReportManager.getOvdRate(last30Date, lastDate, planBase.getPlanNo()));
                    item.setIdleRate(analyseSpvReportManager.getAvgIdleRateRate(lastDate, planBase));
                    item.setRepayRate(analyseSpvReportManager.getRepayRate(last30Date, lastDate, planBase.getPlanNo()));
                    item.setPreRepayRate(analyseSpvReportManager.getPreRepayRate(last30Date, lastDate, planBase.getPlanNo()));
                    item.setFundPlanDate(file == null ? "--" : DateUtil.formatDate(file.getUploadDate()));
                }
                items.add(item);
            });

            page.setDatas(items);
            page.setTotal(planList.getTotal());
            page.setCurrPageNo(planList.getCurrPageNo());
            return page;
        });
    }
    
    /**
     * 查询计划概要信息
     * @param param
     * @return
     */
    @Override
    public PlanAnalyseVO getAnalyseSpvPlan(AnalyseBaseParam param) {
        return super.doQry(param, () -> {
            VarChecker.checkNotEmpty(param.getPlanNo(), "项目编号不能为空");
            // 转让系统会计日
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            // 查询计划
            PlanBase planBase = analyseSpvReportManager.checkGetPlan(param.getPlanNo());
            // 最后一日报表
            PlanReport planReport = planReportRepo.getLatestReport(param.getPlanNo(), ReportTypeEnum.RPT001);
            // 统计日 默认D-1日
            Date accDate = AnalyseReportUtil.getStatDate(trfDate, param.getAccountDate(), planReport);

            PlanAnalyseVO analyseVO = new PlanAnalyseVO();
            analyseVO.setPlanNo(planBase.getPlanNo());
            analyseVO.setPlanName(planBase.getPlanName());
            // 查询计划本金规模  "WK017", "总贷款本金余额"
            Map<String, PlanReportItem> itemMap = analyseSpvReportManager.querySpvStatPlanDayItem(
                    accDate, param.getPlanNo(), CollectionUtil.newArrayList(WkAstReportFields.WK017), true);
            analyseVO.setPrinScale(AnalyseReportUtil.getItemMoney(itemMap, WkAstReportFields.WK017));
            analyseVO.setAccountDate(DateUtil.formatDate(accDate));
            return analyseVO;
        });
    }
    
}
