/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.api.plan;

import com.abssqr.plat.biz.shared.handler.PlanAllocHandler;
import com.abssqr.plat.biz.support.ServiceSupport;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanAllocStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocHandleParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocPayParam;
import com.abssqr.plat.common.facade.result.plan.PlanAllocPayInfoVO;
import com.abssqr.plat.common.facade.service.plan.PlanAllocPayService;
import com.abssqr.plat.common.facade.validation.Valid;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * 分配支付或入账
 *
 * @author hanfei
 * @version $Id: PlanAllocPayServiceImpl.java, v 0.1 2019-07-09 4:48 PM hanfei Exp $
 */
@Service
public class PlanAllocPayServiceImpl extends ServiceSupport implements PlanAllocPayService {

    @Autowired
    private PlanAllocRepo    planAllocRepo;
    @Autowired
    private PlanAllocHandler planAllocHandler;
    @Autowired
    private PlanRepo         planRepo;
    @Autowired
    private AcctManager      acctManager;

    /**
     * 分配支付或人工分配入账
     *
     * @param param
     */
    @Override
    public void allocPay(@Valid PlanAllocPayParam param) {
        super.doOpr(param, () -> {
            // 获取对应分配计划进行支付
            PlanAlloc alloc = planAllocRepo.getPlanAllocDetail(param.getAllocNo());
            if (alloc.getStatus() == PlanAllocStatusEnum.ASSIGNED) {
                return null;
            }
            PlanAllocHandleParam handleParam = new PlanAllocHandleParam();
            handleParam.setAlloc(alloc);
            handleParam.setOutTransNo(param.getOutTransNo());
            handleParam.setOffLine(param.getOffLine());
            // 判断线上线下操作
            if (param.getOffLine()) {
                // 线下支付 创建支付单，同时进行入账等后续操作
                planAllocHandler.execute(handleParam);
            } else {
                // 在线支付，只创建支付单
                planAllocHandler.handle(handleParam);
            }
            return null;
        });

    }

    /**
     * 查询分配详情
     *
     * @param param
     * @return
     */
    @Override
    public PlanAllocPayInfoVO allocInfo(@Valid PlanAllocPayParam param) {
        return super.doQry(param, () -> {
            // 获取对应分配计划进行支付
            PlanAlloc alloc = planAllocRepo.getPlanAllocDetail(param.getAllocNo());
            // 获取对应计划
            PlanBase plan =  planRepo.getPlanByNo(alloc.getPlanNo());
            //回款账户状态
            AccountAttributeEnum sourceAcct = PlanTypeEnum.SPV == alloc.getPlanType() ?
                                                      AccountAttributeEnum.RECV : AccountAttributeEnum.NORMAL;

            // 获取账户信息
            List<Account> accounts = acctManager.getAllAcctByPlanNo(alloc.getPlanNo(),
                Lists.newArrayList(AccountTypeEnum.BANK, AccountTypeEnum.PLAT_VIRTUAL));
            VarChecker.checkNotEmpty(accounts, "计划[{0}]找不到对应的账户信息", alloc.getPlanNo());
            // 组装模型
            PlanAllocPayInfoVO infoVO = new PlanAllocPayInfoVO();
            infoVO.setPlanNo(alloc.getPlanNo());
            infoVO.setPlanName(plan.getPlanName());
            infoVO.setPayAmt(alloc.getTotalConfirmAllocAmt());
            for (Account account : accounts) {
                // 源账户 为 银行一般业务户
                if (account.getActAttr() == sourceAcct
                    && account.getActType() == AccountTypeEnum.BANK) {
                    infoVO.setSourceAcct(infoVO.installAccount(account));
                }
                // 目标账户为 银行 托管户
                if (account.getActAttr() == AccountAttributeEnum.HOSTED
                    && account.getActType() == AccountTypeEnum.BANK) {
                    infoVO.setTargetAcct(infoVO.installAccount(account));
                }
            }
            return infoVO;
        });
    }

}
