/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.planAstStat;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.biz.shared.handler.PlanAstClearingStatHandler;
import com.abssqr.plat.common.dal.mysql.auto.dao.PlanAstStatTaskDAO;
import com.abssqr.plat.common.facade.enums.OrgAstStatStatusEnum;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 计划统计任务
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.task.task: PlanAstStatTaskExecutor.java, v 0.1 2018-01-08 02:15 hongwei.wang Exp $
 */
@Component
public class PlanAstStatTaskExecutor extends DispatcherExecutorTemp<PlanAstStatTask> {
    private final static Logger logger       = LoggerFactory.getLogger(
            PlanAstStatTaskExecutor.class);
    
    @Autowired
    private PlanAstClearingStatHandler planAstClearingStatHandler;
    @Autowired
    private PlanAstStatTaskDAO planAstStatTaskDAO;
    
    @Override
    protected PlanAstStatTask doExecute(PlanAstStatTask node) {
        String execTms = TimeStampUtil.getCurrentBufferedTimestamp();
        return planAstClearingStatHandler.execute(node, execTms);
    }
    
    @Override
    protected PlanAstStatTask markSuccess(PlanAstStatTask node) {
        // 在内部处理，不单独标记
        return node;
    }
    
    @Override
    protected PlanAstStatTask markFail(PlanAstStatTask node, String failMessage) {
        // 因异常引起的重试
        Date nextTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 30);
        node.setStatus(OrgAstStatStatusEnum.ACTIVE);
        planAstStatTaskDAO.updateTaskExep(nextTime, node.getPlanNo());
        return node;
    }
    
    @Override
    protected PlanAstStatTask markException(PlanAstStatTask node, String failMessage) {
        Date nextTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 30);
        node.setStatus(OrgAstStatStatusEnum.ACTIVE);
        planAstStatTaskDAO.updateTaskExep(nextTime, node.getPlanNo());
        return node;
    }
    
    @Override
    protected String getName() {
        return "PlanAstStatTaskExecutor";
    }
}
