/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.planAstStat;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 资产转让
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.task.task: PlanAstStatTaskLoader.java, v 0.1 2018-01-08 00:59 hongwei.wang Exp $
 */
@Component
public class PlanAstStatTaskLoader implements DispatcherLoader<PlanAstStatTask> {
    @Autowired
    private PlanAstStatTaskRepo planAstStatTaskRepo;
    @Autowired
    private TransactionTemplate transactionTemplate;
    @Autowired
    private HostManager         hostManager;
    @Autowired
    private JobControlRepository jobControlRepository;
    
    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<PlanAstStatTask> loadToDoNodes(int size) {
        
        return transactionTemplate.execute(transactionStatus -> {
    
            // 太多乐观锁冲突报错了，还是加个锁吧
            jobControlRepository.lockByJobName(PlanAstStatTaskStarter.JOB_NAME);
            
            List<PlanAstStatTask> tasks = planAstStatTaskRepo.getToDo(SystemDateUtil.getSystemDate(), size);
            if (CollectionUtils.isEmpty(tasks)) {
                return Lists.newArrayList();
            }
            
            List<String> planNos = tasks.stream().map(PlanAstStatTask::getPlanNo).collect(Collectors.toList());
            int count = planAstStatTaskRepo.updateTaskExec(SystemDateUtil.getSystemDate(),
                    hostManager.getHostInfo().getHostName(),
                    planNos);
            VarChecker.checkEquals(count, planNos.size(), "[TaskLoader.loadToDoNodes]更新任务数跟期望数不一致，实际更新：{0}，期望更新：{1} - {2}",
                    count,
                    planNos.size(),
                    planNos);
            return tasks;
        });
    }
}
