/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkPlanReport;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherExecutorTemp;
import com.abssqr.plat.biz.shared.handler.WkPlanReportHandler;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.SystemDateUtil;
import org.apache.commons.lang3.tuple.MutablePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

/**
 * 计划统计任务
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.task.task: PlanAstStatTaskExecutor.java, v 0.1 2018-01-08 02:15 hongwei.wang Exp $
 */
@Component
public class PlanReportTaskExecutor extends DispatcherExecutorTemp<CommonTask> {
    private final static Logger LOGGER = LoggerFactory.getLogger(
            PlanReportTaskExecutor.class);
    
    @Autowired
    private WkPlanReportHandler    wkPlanReportHandler;
    @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;
    @Autowired
    private CommonTaskRepository   commonTaskRepository;
    @Autowired
    private PlanManager            planManager;
    
    @Override
    public CommonTask doExecute(CommonTask node) {
        LogUtil.info(LOGGER, "执行计划报表任务[{0}]", node);
        PlanBase planBase = planManager.getPlanFullByNo(node.getContext());
        Day accountDate = orgAcctDayComponent.getAstAcctDay(orgAcctDayComponent.getOrgCode(planBase));
    
        MutablePair<Boolean, String> rptResult = wkPlanReportHandler.execute(accountDate.getStandardDate(), planBase);
        if (rptResult.getLeft()) {
            return markSuccess(node);
        } else {
            return markFail(node, MessageUtil.formatMsg("[{0}]项目[{1}]报表执行暂时退出:{2}",
                    planBase.getPlanNo(), accountDate.getDayString(), rptResult.getRight()));
        }
    }
    
    @Override
    protected CommonTask markSuccess(CommonTask node) {
        commonTaskRepository.updateTaskStatus(node.getId(), TaskStatusEnum.SUCCESS, TaskStatusEnum.EXECUTING, "");
        node.setTaskStatus(TaskStatusEnum.SUCCESS);
        return node;
    }
    
    @Override
    protected CommonTask markFail(CommonTask node, String failMessage) {
        // 因异常引起的重试
        Date nextTime = DateTimeUtil.addSecond(SystemDateUtil.getSystemDate(), 60);
        String msg = failMessage.length()>2000?failMessage.substring(0,2000):failMessage;
        commonTaskRepository.updateTaskExep(nextTime, node.getId(), msg);
        return node;
    }
    
    @Override
    protected CommonTask markException(CommonTask node, String failMessage) {
        return markFail(node, failMessage);
    }
    
    @Override
    protected String getName() {
        return "PlanReportTaskExecutor";
    }
}
