/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.dispatcher.wkPlanReport;

import com.abssqr.plat.biz.shared.dispatcher.DispatcherLoader;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.host.HostManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 计划报表异步处理
 *
 * @author zhenxuan.luo
 * @version com.abssqr.piz.shared.task.task: PlanAstStatTaskLoader.java, v 0.1 2018-01-08 00:59 hongwei.wang Exp $
 */
@Component
public class PlanReportTaskLoader implements DispatcherLoader<CommonTask> {
    
    @Autowired
    private TransactionTemplate    transactionTemplate;
    @Autowired
    private HostManager            hostManager;
    @Autowired
    private JobControlRepository   jobControlRepository;
    @Autowired
    private PlanRepo               planRepo;
    @Autowired
    private PlanReportRepo         planReportRepo;
    @Autowired
    private CommonTaskRepository   commonTaskRepository;
    @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;
    
    /**
     * 装载待分发节点
     *
     * @param size
     * @return
     */
    @Override
    public List<CommonTask> loadToDoNodes(int size) {
        
        List<CommonTask> tasks = Lists.newArrayList();
    
        List<CommonTask> existsTask = commonTaskRepository
                                              .selectUnifinishTaskByType(CommonTaskTypeEnum.PLAN_REPORT);
        Map<String, CommonTask> taskMap = existsTask.stream()
                                                  .collect(Collectors.toMap(CommonTask::getContext, item -> item));
        List<PlanBase> plans = planRepo.getPlansByStatus(null, PlanStatusEnum.ONGOING);
        
        return transactionTemplate.execute(transactionStatus -> {
            
            // 太多乐观锁冲突报错了，还是加个锁吧
            jobControlRepository.lockByJobName(PlanReportTaskStarter.JOB_NAME);
            
            for (PlanBase planBase : plans) {
                CommonTask commonTask = taskMap.get(planBase.getPlanNo());
                // 获取对应机构会计日
                Day accountDate = orgAcctDayComponent.getAstAcctDay(orgAcctDayComponent.getOrgCode(planBase));
                if (null == commonTask) {
                    PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
                    DailyReportDateField reportDateField;
                    
                    if (null != planReport) {
                        // 只判断日报，月报没有判断，当天月报失败需要第二天才能执行 或手动触发
                        reportDateField = (DailyReportDateField) planReport.getRptDate();
                        if (DateTimeUtil.calcDayDiff(reportDateField.getAccountDate(), accountDate.getStandardDate()) >= 0) {
                            continue;
                        }
                    }
                    
                    commonTask = new CommonTask();
                    commonTask.setBizNo(SequenceUtil.genWkSeqNo());
                    commonTask.setTaskType(CommonTaskTypeEnum.PLAN_REPORT);
                    commonTask.setContext(planBase.getPlanNo());
                    commonTask.setEnv(null);
                    commonTask.setMemo("");
                    commonTask.setTaskStatus(TaskStatusEnum.INIT);
                    commonTask.setNextExecTime(SystemDateUtil.getSystemDate());
                    commonTask.setExecTimes(0L);
                    commonTaskRepository.create(commonTask);
                    
                } else {
                    if (commonTask.getTaskStatus() != TaskStatusEnum.INIT) {
                        continue;
                    } else if (SystemDateUtil.getSystemDate().compareTo(commonTask.getNextExecTime()) < 0) {
                        continue;
                    }
                }
                
                tasks.add(commonTask);
                
                if (tasks.size() >= size) {
                    break;
                }
            }
            
            if (CollectionUtils.isNotEmpty(tasks)) {
                commonTaskRepository
                        .updateTaskExec(SystemDateUtil.getSystemDate(), hostManager.getHostInfo().getHostName(),
                                tasks);
            }
            return tasks;
        });
    }
}
