/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.handler;

import cn.hutool.core.util.StrUtil;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanClearingOrderItem;
import com.abssqr.plat.common.facade.enums.AstFlowTypeEnum;
import com.abssqr.plat.common.facade.enums.OrgAstStatStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.facade.model.trans.CapitalTrader;
import com.abssqr.plat.common.facade.model.trans.ChangeStageParam;
import com.abssqr.plat.common.model.domain.clearing.SummaryClearingOrder;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.utils.DayUtil;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.share.pay.PlanCapitalCreateRequest;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.model.BaseEntity;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import org.apache.commons.lang3.tuple.MutablePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * 根据结算单 创建 交易支付单
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.handler: PlanAstClearingOrderHandler.java, v 0.1 2019-07-15 01:57 zhenxuan.luo Exp $
 */
@Component
public class PlanAstClearingOrderHandler {

    private final static Logger LOGGER = LoggerFactory.getLogger(PlanAstClearingOrderHandler.class);

    @Autowired
    private PlanClearingOrderRepo   planClearingOrderRepo;
    @Autowired
    private PlanAstStatTaskRepo     planAstStatTaskRepo;
    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private PlanAstManager          planAstManager;
    @Autowired
    private PlanRepo                planRepo;
    /**
     * 清算 根据结算单生成支付交易单处理
     * 事务在任务调用处进行开启
     *
     * @param entity
     */
    public void handle(PlanBase entity) {
        // 资产统计任务加锁  判断为等待状态
        PlanAstStatTask task = planAstStatTaskRepo.lock(entity.getPlanNo());
        if (task == null || task.getStatus() != OrgAstStatStatusEnum.WAIT) {
            return;
        }
        // 拉取待支付的 结算单TaskStatusEnum.INIT
        List<PlanClearingOrderItem> orderItemStats =
                planClearingOrderRepo.queryByClearingOrderStat(task.getPlanNo(), TaskStatusEnum.INIT, null);
        if (ToolUtil.isEmpty(orderItemStats)) {
            LogUtil.info(LOGGER, "未找到计划[{0}]待处理结算单", task.getPlanNo());
            return;
        }
        Date accountDate = task.getAccountDate();
        // 汇总结算单
        List<SummaryClearingOrder> sumOrders = this.calcSummary(orderItemStats);

        /**
         * 放款池 放款终止日 停止支付标记
         * true 代表停止支付直接更新结算单状态
         */
        boolean stopPay = this.stopPaySign(entity, accountDate);
        // 支付交易单计数
        AtomicInteger counter = new AtomicInteger();
        Map<String, PlanBase> cntPtyMap = new HashMap<>();
        // 更新水位金额变化 及登记流水
        planAstManager.changePlanAst(accountDate, entity.getPlanNo(), (planAst, planAstLogList) -> {
            // 记录清算在资产水位表中 待结算的收入支出金额变化
            // 根据汇总后的结算单 创建交易支付单据
            for (SummaryClearingOrder order : sumOrders) {
                // 查询对手机构
                PlanBase cntPtyPlan = planRepo.getPlanByMapCache(cntPtyMap, order.getCntPtyCode());
                // 构建交易双方及交易编码
                CapitalTrader capitalTrader = new CapitalTrader(planAst.getPlanNo(), planAst.getPlanType(),
                        cntPtyPlan.getPlanNo(), cntPtyPlan.getPlanType(), order.getTransCode());
                // 获取交易单类型
                PlanCapitalTransTypeEnum transType = capitalTrader.getCapitalTransType();
                /**
                 * 支出时 用水位余额 与 交易金额 计算可支出金额，当金额小于等于0时不创建支付单
                 * 1、支付应付利息时 = 支付应付利息
                 * 2、其他支出时 = 水位可支出金额 > TransAmt ? TransAmt ：水位可支出金额
                 * 返回值 left 可交易金额 right 备注内容
                 */
                MutablePair<Money, String> ableTransInfo = this.calcCapitalTransAmt(transType, planAst, order, stopPay);

                // 1. 根据实际交易金额与结算单金额 创建支付交易订单 capitalOrder   可交易金额大于零 创建支付单  小于等于零创建his备份
                PlanCapitalOrder capitalOrder = this.createCapitalOrder(transType, order, ableTransInfo.getLeft(), counter);
    
                // 2. 更新计划资产水位表 在途的支出收入金额 并登记日志
                planAstLogList.addAll(planAstManager.execChangeStageTwoWay(planAst,
                        ChangeStageParam.createParamByCapital(capitalTrader, StageChangeTypeEnum.ON_WAY, capitalOrder)));

                // 3. 修改结算单状态为 SUCCESS，并将capitalOrder交易单的seqNo 回填至ClearingOrder
                this.modifyClearingOrderStatus(order, ableTransInfo.getRight().concat(
                        MessageUtil.formatMsg("-支付单seqNo[{0}]", capitalOrder.getSeqNo())));
            }
        });
        LogUtil.info(LOGGER, "计划[{0}]处理结算单完成,结算单[{1}]条,创建支付订单[{2}]条",
                task.getPlanNo(), orderItemStats.size(), counter.get());
    }


    /**
     * 更新结算单状态及回填交易单编号
     *
     * @param order      结算单
     * @param memo       备注
     */
    private void modifyClearingOrderStatus(SummaryClearingOrder order, String memo) {
        LogUtil.info(LOGGER, memo);
        // 根据 初始化状态 的结算单进行更新状态 并将汇总交易编号更新到结算单上
        int count = planClearingOrderRepo.updateOrderTransBySeqNos(memo, order.getSeqNos(),
                TaskStatusEnum.SUCCESS, order.getTransBizNo(), TaskStatusEnum.INIT);
        // 乐观锁控制 是否存在并发处理
        if (count != order.getSeqNos().size()) {
            String msg = MessageUtil.formatMsg("计划[{0}-{1}]结算单变更[{0}]和任务数[{1}]不一致,等待重新执行",
                    count, order.getSeqNos().size());
            throw new AbssqrBizException(msg, true);
        }
    }

    /**
     * 判断当前水位能否进行支付交易 不进行支付交易时返回金额0
     *
     * @param planAst
     * @param order
     * @param stopPay
     * @return
     */
    private MutablePair<Money, String> calcCapitalTransAmt(PlanCapitalTransTypeEnum transType, PlanAst planAst,
                                                           SummaryClearingOrder order, boolean stopPay) {
        // 返回可交易金额 及 备注
        MutablePair<Money, String> pair = new MutablePair(new Money(), "");
        /*
         承接池
             AST_REPO("PCT02", "资产回收兑付"
             BUY_AST("PCT03", "资产交易兑付"
             GRANT_REVERSE("PCT05", "放款冲正"
            -- PAY_HANG_INTR("PCT07", "支付应付利息"
         放款池 - 回转放业务
            NT_AST_REPO("PCT11", "未转资产回收划账"
            NT_BUY_AST("PCT12", "资产交易兑付划账"
           --  NT_PAY_HANG_INTR("PCT14", "应付利息兑付回收划账"
         */
        // 判断交易对手一致 为放款池回转放业务 进行可支付余额计算 PCT11 PCT12
        if (StrUtil.equals(order.getPlanNo(), order.getCntPtyCode())) {
            VarChecker.checkArgument(transType == PlanCapitalTransTypeEnum.NT_AST_REPO
                                     || transType == PlanCapitalTransTypeEnum.NT_BUY_AST
                                     || transType == PlanCapitalTransTypeEnum.NT_PAY_HANG_INTR,
                    "{0}[{1}]不支持的交易类型,交易码[{2}],对手方机构号[{3}]", planAst.getPlanType().getCode(),
                    planAst.getPlanNo(), order.getTransCode().getCode(),order.getCntPtyCode());
            if (stopPay) {
                // 停止支付时 直接更新结算单状态 不进行支付操作
                pair.setRight("放款终止日之后不进行结算支付操作");
                return pair;
            }
            // 水位可支出金额
            Money ablePayCash = planAst.ablePayCash().clone();
            // 可支出金额 当前水位可用支付余额
            Money ablePayAmt = ablePayCash.subtract(order.getTransAmt());
            //【水位可用支付余额 小于 交易金额】? 用水位可用支付金额 : 实际交易金额
            pair.setLeft(ablePayAmt.isNegtive() ? ablePayCash : order.getTransAmt().clone());
            // 水位余额不足 不进行支付操作
            if (!pair.getLeft().isGreaterThanZero()) {
                pair.setRight(MessageUtil.formatMsg("{0}[{1}]水位不足,TransCode[{2}],amt[{3}],cash[{4}],可用余额[{5}]",
                        planAst.getPlanType().getCode(), planAst.getPlanNo(), order.getTransCode().getCode(),
                        order.getTransAmt(), planAst.getCash(), ablePayCash));
            }
            // 实际支付金额小于应付金额 将差额追加到校验金额上满足后续变更水位校验
            Money diffAmt = order.getTransAmt().subtract(pair.getLeft());
            if (diffAmt.isGreaterThanZero()){
                planAstManager.changeCheckCashAmt(planAst, diffAmt, AstFlowTypeEnum.FLOW_IN);
            }
        } else {
            // 其他情况直接为交易金额
            pair.setLeft(order.getTransAmt().clone());
        }
        return pair;
    }

    /**
     * 创建支付交易单
     *
     * @param transType    交易单类型
     * @param order        结算单
     * @param ableTransAmt 实际交易金额
     * @param count        计数器
     * @return
     */
    private PlanCapitalOrder createCapitalOrder(PlanCapitalTransTypeEnum transType, SummaryClearingOrder order,
                                                Money ableTransAmt, AtomicInteger count) {
        // 交易类型
        PlanCapitalCreateRequest request = new PlanCapitalCreateRequest();
        request.setBizNo(order.getTransBizNo());
        request.setTransType(transType);
        request.setPlanNo(order.getPlanNo());
        request.setTargetOrgCode(order.getCntPtyCode());
        request.setAccountDate(order.getAccountDate());
        // 实际交易金额
        request.setTransAmt(ableTransAmt);
        // 结算单汇总金额为 应付金额
        request.setPayableAmt(order.getTransAmt());
        // 返回支付交易单
        PlanCapitalOrder capitalOrder = planCapitalOrderManager.createCapitalOrder(request);
        LogUtil.info(LOGGER, "{0}[{1}->{2}]创建支付交易单type[{3}]amt[{4}]CapitalOrderSeqNo[{5}]",
                order.getAccountDate(), order.getPlanNo(), order.getCntPtyCode(), transType, ableTransAmt, capitalOrder.getSeqNo());
        // 订单数 +1
        count.getAndIncrement();
        return capitalOrder;
    }

    /**
     * 判断放款池的支付标记
     *
     * @param planBase
     * @param accountDate
     * @return
     */
    private boolean stopPaySign(PlanBase planBase, Date accountDate) {
        // 如果是放款池计划 进行冻结预留及 判断转账限制
        if (planBase.getPlanType() == PlanTypeEnum.SPV && planBase instanceof OrganizationEntity) {
            OrganizationEntity entity = (OrganizationEntity) planBase;
            // 计算当前日期与 放款截止日相差天数
            int dayDiff = DayUtil.diffDays(new Day(accountDate), new Day(entity.getStopPayDate()));
            // 放款截止日后 不进行 回款户向 放款户资金划账打款操作
            if (dayDiff >= 0) {
                // 直接标记结算单的成功
                return true;
            }
        }
        return false;
    }

    /**
     * 汇总计算
     *
     * @param orderStats
     * @return
     */
    private List<SummaryClearingOrder> calcSummary(List<PlanClearingOrderItem> orderStats) {
        // 按照会计日 交易码 交易对手进行分组汇总数据
        Map<ClearingOrderKey, List<PlanClearingOrderItem>> orderStatMap =
                orderStats.stream().collect(Collectors.groupingBy(
                        item -> new ClearingOrderKey(item.getAccountDate(),
                                TransCodeEnum.getByCode(item.getTransCode()), item.getCntPtyCode())));
        // 进行汇总
        List<SummaryClearingOrder> summaryList = new ArrayList(orderStatMap.size());
        orderStatMap.keySet().forEach(orderKey -> {
            SummaryClearingOrder order = new SummaryClearingOrder();
            orderStatMap.get(orderKey).stream().forEach(item -> {
                order.setPlanNo(item.getPlanNo());
                order.setTransCode(TransCodeEnum.getByCode(item.getTransCode()));
                order.setCntPtyCode(item.getCntPtyCode());
                order.setAccountDate(item.getAccountDate());
                order.getTransAmt().addTo(item.getTransAmt());
                order.getSeqNos().add(item.getSeqNo());
                // 汇总交易编码
                String transBizNo = SequenceUtil.genSeqNo(order.getTransCode());
                order.setTransBizNo(transBizNo);
            });
            summaryList.add(order);
        });
        return summaryList;
    }

    /**
     * 分组 key
     */
    private class ClearingOrderKey extends BaseEntity {
        /**
         * 交易日期
         */
        Date          accountDate;
        /**
         * 交易对手
         */
        String        cntyPtyCode;
        /**
         * 交易码
         */
        TransCodeEnum transCode;

        public ClearingOrderKey(Date accountDate, TransCodeEnum transCode, String cntyPtyCode) {
            this.cntyPtyCode = cntyPtyCode;
            this.transCode = transCode;
            this.accountDate = accountDate;
        }
    }

    //        //  【原有轧差方案】  3、根据对手机构进行汇总支付与收入，进行轧差计算，创建支付订单，一个对手机构对应一笔支付交易单
    //        for (Map.Entry<ClearingOrderKey, List<PlanClearingOrderItem>> stat : orderStatMap.entrySet()) {
    //            ClearingOrderKey orderKey = stat.getKey();
    //            // 结算单
    //            List<PlanClearingOrderItem> orderList = stat.getValue();
    //            List<String> seqNos = orderList.stream().map(PlanClearingOrderItem::getSeqNo).collect(Collectors.toList());
    //
    //            // 轧差计算金额 是支出还是收入，同时更新计划资产水位表，追加在途的支出收入金额，扣减待结算支出收入金额 并登记日志
    //            MutablePair<PlanCapitalTransTypeEnum, Money> pair = this.calcNetting(orderKey.accountDate, orgEntityCode,
    //                    orderList);
    //            if (pair.getRight().isZero()) {
    //                // 收支平衡 没有支付动作，将结算单全部更新成功
    //                modifyOrderStatus(TaskStatusEnum.SUCCESS, "", seqNos);
    //            } else {
    //                // 4、 创建支付订单
    //                String bizNo = createCapitalOrder(pair.getLeft(), orgEntityCode, orderKey.cntyPtyCode,
    //                        pair.getRight(), orderKey.accountDate);
    //                count++;
    //                LogUtil.info(LOGGER, "计划[{0}-{1}]创建支付交易单type[{2}]cntPty[{3}]amt[{4}]bizNo[{5}]",
    //                        orgEntityCode, orderKey.accountDate, pair.getLeft(), orderKey.cntyPtyCode, pair.getRight(),
    //                        bizNo);
    //                // 5、修改结算单状态为 WAITING，等待支付结果
    //                modifyOrderStatus(TaskStatusEnum.WAITING, bizNo, seqNos);
    //            }
    //        }


    //    /**
    //     * 轧差计算金额，同时更新计划资产水位表，追加在途的支出收入金额，扣减待结算支出收入金额 并登记日志
    //     * @param orderList
    //     * @return MutablePair <收支类型, 汇总金额>
    //     */
    //    @Deprecated
    //    private MutablePair<PlanCapitalTransTypeEnum, Money> calcNetting(Date accountDate, String planNo,
    //                                                                     List<PlanClearingOrderItem> orderList) {
    //
    //        final Money netAmt = new Money();
    //        final MutablePair<PlanCapitalTransTypeEnum, Money> triple = new MutablePair<>(null, new Money());
    //
    //        // 6、记录清算在资产水位表中 待结算的收入支出金额变化
    //        planAstManager.changePlanAst(accountDate, planNo, planAst -> {
    //
    //            List<PlanAstLog> planAstLogList = new ArrayList<>();
    //
    //            for (PlanClearingOrderItem order : orderList) {
    //                TransCodeEnum transCode = TransCodeEnum.getByCode(order.getTransCode());
    //                // 流入流出类型
    //                AstFlowTypeEnum flowTypeEnum = transCode.getFundFlow();
    //
    //                netAmt.addTo(order.getTransAmt().multiply(flowTypeEnum.getSign()));
    //
    //                // 轧差模式，只登记明细的流水，不做实际发生。最后用轧差后的金额进行实际发生
    //                PlanAstLog planAstLog = PlanAstLog.buildAstLog(planAst, accountDate, transCode,
    //                        StageChangeTypeEnum.NO_CHANGE);
    //                // 无类型发生额
    //                planAstLog.setHappenAmt(order.getTransAmt());
    //                planAstLogList.add(planAstLog);
    //            }
    //
    //            TransCodeEnum transCode;
    //            if (triple.getRight().isGreaterThanZero()) {
    //                // 大于 0 代表收入
    //                triple.setLeft(PlanCapitalTransTypeEnum.AST_REPO);
    //                transCode = TransCodeEnum.REPO;
    //            } else {
    //                // 支出
    //                triple.setLeft(PlanCapitalTransTypeEnum.BUY_AST);
    //                transCode = TransCodeEnum.BUY_AST;
    //                // 金额转正
    //                netAmt.multiplyBy(-1L);
    //            }
    //
    //            triple.setRight(netAmt);
    //
    //            PlanAstLog planAstLog = PlanAstLog.buildAstLog(planAst, accountDate, transCode, StageChangeTypeEnum.ON_WAY);
    //            // 更新计划资产水位表，追加在途的支出收入金额，扣减待结算支出收入金额 并登记日志
    //            // 根据 TransCodeEnum 与 StageChangeTypeEnum 进行变更
    //            planAstLog.changeStageAndLog(planAst, netAmt, netAmt, "");
    //            planAstLogList.add(planAstLog);
    //            return planAstLogList;
    //        });
    //
    //        return triple;
    //    }
}