/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.handler;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.biz.shared.listener.PlanMonitorRecordEvent;
import com.abssqr.plat.biz.shared.report.WkPlanReportChecker;
import com.abssqr.plat.biz.shared.report.wk.WkAstReportBuilder;
import com.abssqr.plat.biz.shared.report.wk.WkManagerReportBuilder;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanAstReportStat;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.PlanStatItem;
import com.abssqr.plat.common.facade.enums.EventCodeEnum;
import com.abssqr.plat.common.facade.enums.ReportTitleGroupEnum;
import com.abssqr.plat.common.facade.enums.ReportTypeEnum;
import com.abssqr.plat.common.facade.enums.TitleTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAcctTitle;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.report.DailyReportDateField;
import com.abssqr.plat.common.facade.model.report.MonthReportDateField;
import com.abssqr.plat.common.facade.model.report.PlanReport;
import com.abssqr.plat.common.facade.model.report.PlanReportItem;
import com.abssqr.plat.common.facade.model.report.wk.WkManagerReport;
import com.abssqr.plat.common.model.domain.plan.PlanAcctTitleLog;
import com.abssqr.plat.common.model.enums.IdTypeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleLogRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAcctTitleRepo;
import com.abssqr.plat.common.model.repo.plan.PlanAstLogRepository;
import com.abssqr.plat.common.model.repo.report.PlanReportEntityRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportEntityStatRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.core.service.report.LoanBakStatManager;
import com.alibaba.fastjson.JSON;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.tuple.MutablePair;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Component;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 计划报表统计任务
 *
 * @author zhenxuan.luo
 * @version com.abssqr.plat.biz.shared.scheduler.job.plan: PlanPackGenJob.java, v 0.1 2019-06-18 21:19 zhenxuan.luo
 * Exp $
 */
@Component
public class WkPlanReportHandler {
    
    protected final Logger                   LOGGER = LoggerFactory.getLogger(getClass());
    @Autowired
    private ApplicationContext       applicationContext;
    @Autowired
    private PlanReportEntityRepo     planReportEntityRepo;
    @Autowired
    private PlanAstLogRepository     planAstLogRepository;
    @Autowired
    private TransactionTemplate      transactionTemplate;
    @Autowired
    private WkAstReportBuilder       wkAstReportBuilder;
    @Autowired
    private WkManagerReportBuilder   wkManagerReportBuilder;
    @Autowired
    private PlanReportRepo           planReportRepo;
    @Autowired
    private PlanReportEntityStatRepo planReportEntityStatRepo;
    @Autowired
    private WkPlanReportChecker      wkPlanReportChecker;
    @Autowired
    private PlanAcctTitleLogRepo     planAcctTitleLogRepo;
    @Autowired
    private PlanAcctTitleRepo        planAcctTitleRepo;
    @Autowired
    private LoanBakStatManager       loanBakStatManager;
    /**
     * 执行报表
     * @param accountDate
     * @param planBase
     * @return
     */
    public MutablePair<Boolean, String> execute(Date accountDate, PlanBase planBase) {
        // 日报组装
        MutablePair<Boolean, String> resultRptPair = checkAndBuildDailyAstReport(accountDate, planBase);
        if (resultRptPair.getLeft()){
            // 月报组装
            checkAndBuildMonthPlanReport(accountDate, planBase);
        }
        return resultRptPair;
    }
    
    /**
     * 日报表组装
     *
     * @param accountDate
     * @param planBase
     */
    public MutablePair<Boolean, String> checkAndBuildDailyAstReport(Date accountDate, PlanBase planBase) {
        // 查询最近的报表，若存在，则直接取下一个周期进行生成，避免中间因为系统故障导致报表没有生成的问题。
        PlanReport planReport = planReportRepo.getLatestReport(planBase.getPlanNo(), ReportTypeEnum.RPT001);
        // 日期控制对象
        DailyReportDateField reportDateField = new DailyReportDateField();
        // 放款池计划时 对应的承接池报表编号
        List<String> planNos = Lists.newArrayList();
        // 校验报表是否可执行
        MutablePair<Boolean, String> rptResultPair =
                wkPlanReportChecker.checkDailyAstReport(accountDate, planBase, planReport, reportDateField, planNos);
        if (!rptResultPair.getLeft()){
            return rptResultPair;
        }
        // 组装报表数据
        buildDailyAstReport(reportDateField, planBase, planReport, planNos);
        return rptResultPair;
    }
    
    /**
     * 组装报表数据
     * @param reportDateField 日期控制
     * @param planBase   计划
     * @param lastReport    最后一日报表
     * @param planNos       放款池时对应承接池计划编号
     */
    public void buildDailyAstReport(DailyReportDateField reportDateField, PlanBase planBase,
                                                            PlanReport lastReport, List<String> planNos) {
        
        LogUtil.info(LOGGER, "[ReportJob]-[{0}]计划在[{1}]-[{2}]报表生成任务执行*开始*",
                planBase.getPlanNo(), reportDateField.getAccountDate(), ReportTypeEnum.RPT001);
        //余额项统计  使用loanBak统计余额
        List<PlanAstReportStat> statLoanBakBal =
                loanBakStatManager.statLoanBakBal(reportDateField.getAccountDate(), planBase);
        //发生额项统计
        List<PlanAstReportStat> statAmt =
                planReportEntityRepo.statAmt(reportDateField.getAccountDate(), planBase.getPlanNo());
        //每日结息项统计
        List<PlanAstReportStat> statAccrued =
                planReportEntityRepo.statAccrued(reportDateField.getAccountDate(), planBase.getPlanNo());
        //资金项统计-->水位变更
        List<PlanStatItem> planStatItems =
                planAstLogRepository.getAstLogStat(reportDateField.getAccountDate(), planBase.getPlanNo());
        // 获取需要统计的科目流水
        Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap =
                this.getTitleLogMap(reportDateField.getAccountDate(), planBase.getPlanNo());
        
        transactionTemplate.execute(status -> {
            // 组装日报
            List<PlanReportItem> planReportItems = wkAstReportBuilder.buildReport(planBase,
                    lastReport, reportDateField, statLoanBakBal, statAmt, statAccrued, planStatItems, acctTitleLogMap, planNos);
            
            PlanReport newReport = new PlanReport();
            newReport.setRptNo(SequenceUtil.genId(IdTypeEnum.SEQ));
            newReport.setPlanNo(planBase.getPlanNo());
            newReport.setPlanName(planBase.getPlanName());
            newReport.setRptDate(reportDateField);
            newReport.setRptType(ReportTypeEnum.RPT001);
            newReport.addReportItems(planReportItems);
            
            // 创建报表
            planReportRepo.create(newReport);
            
            // 备份资产统计的中间结果，备查
            planReportEntityStatRepo.insertBatch(statLoanBakBal);
            //
            planReportEntityStatRepo.insertBatch(statAmt);
            
            // 数据量较大，定期清理
            //                planReportEntityRepo.delete(reportDateField.getAccountDate(), plan.getPlanNo());
            
            //更新监控指标
            applicationContext.publishEvent(new PlanMonitorRecordEvent(this, newReport, planBase));
            
            return newReport;
        });
        LogUtil.info(LOGGER, "[ReportJob]-[{0}]计划在[{1}]-[{2}]报表生成任务执行*结束*",
                planBase.getPlanNo(), reportDateField.getAccountDate(), ReportTypeEnum.RPT001);
    }
    
    /**
     * 月报组装
     *
     * @param accountDate
     * @param plan
     */
    public MutablePair<Boolean, String> checkAndBuildMonthPlanReport(Date accountDate, PlanBase plan) {
        // 查询最近的报表，若存在，则直接取下一个周期进行生成，避免中间因为系统故障导致报表没有生成的问题。
        PlanReport latestReport = planReportRepo.getLatestReport(plan.getPlanNo(), ReportTypeEnum.RPT002);
    
        MonthReportDateField reportMonthField = new MonthReportDateField();
        // 执行校验
        MutablePair<Boolean, String> rptResultPair =
                wkPlanReportChecker.checkMonthPlanReport(accountDate, plan, latestReport, reportMonthField);
        if (!rptResultPair.getLeft()){
            return rptResultPair;
        }
        // 组装生成月报
        buildMonthPlanReport(reportMonthField, plan, latestReport);
        return rptResultPair;
    }
    
    /**
     * 组装生成月报
     * @param reportMonthField  日期控制
     * @param plan      计划
     * @param latestReport  上一报表
     */
    public void buildMonthPlanReport(MonthReportDateField reportMonthField, PlanBase plan,
                                                             PlanReport latestReport) {
        Date startDate = DateTimeUtil.getMax(plan.getBeginDate(), reportMonthField.getRptStartDate());
        Date endDate = reportMonthField.getAccountDate();
        LogUtil.info(LOGGER, "[buildMonthPlanReport]-[{0}]计划在[{1}]-[{2}]管理报表生成任务执行*开始*[{3}]",
                plan.getPlanNo(), startDate, endDate ,ReportTypeEnum.RPT002 );
        // 上一期报表
        WkManagerReport lastManagerRpt = (latestReport == null) ? null :
                                                 JSON.parseObject(latestReport.getDetail(), WkManagerReport.class);
        // 科目
        List<PlanAcctTitle> planAcctTitles = planAcctTitleRepo.getByEntityNo(plan.getPlanNo());
        // 科目流水
        List<PlanAcctTitleLog> titleLogs = planAcctTitleLogRepo.getByDate(plan.getPlanNo(), startDate, endDate);
        // 生成管理报表
        WkManagerReport curReport = wkManagerReportBuilder.buildReport(
                plan, startDate, endDate, lastManagerRpt, planAcctTitles, titleLogs);
        transactionTemplate.execute(status -> {
            PlanReport newReport = new PlanReport();
            newReport.setRptNo(SequenceUtil.genId(IdTypeEnum.SEQ));
            newReport.setPlanNo(plan.getPlanNo());
            newReport.setPlanName(plan.getPlanName());
            newReport.setRptDate(reportMonthField);
            newReport.setRptType(ReportTypeEnum.RPT002);
            newReport.setDetail(JSON.toJSONString(curReport));
            
            // 创建报表
            planReportRepo.create(newReport);
            return newReport;
        });
        LogUtil.info(LOGGER, "[buildMonthPlanReport]-[{0}]计划在[{1}]-[{2}]管理报表生成任务执行*结束*[{3}]",
                plan.getPlanNo(), startDate, endDate, curReport);
    }
    
    /**
     * 获取 投资结构余额及相关费用的科目流水
     * @param accountDate
     * @param planNo
     * @return
     */
    private Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> getTitleLogMap(Date accountDate, String planNo){
        Map<ReportTitleGroupEnum, List<PlanAcctTitleLog>> acctTitleLogMap = new HashMap<>();
        //金融服务费统计
        PlanAcctTitleLog finFee = planAcctTitleLogRepo.getSettleFinFee(accountDate, planNo);
        acctTitleLogMap.put(ReportTitleGroupEnum.FIN_SER_FEE, Lists.newArrayList(finFee));
    
        // 本金科目
        List<TitleTypeEnum> investedEnums =
                CollectionUtil.newArrayList(TitleTypeEnum.INVESTED, TitleTypeEnum.PRIOR_INVESTED, TitleTypeEnum.SECND_INVESTED);
        // 查询计划投资结构本金分配科目流水
        PlanAcctTitleLog invested = planAcctTitleLogRepo.getAcctTitleByType(accountDate, planNo,
                EventCodeEnum.TITLE_ALLOC, investedEnums);
        acctTitleLogMap.put(ReportTitleGroupEnum.INVESTED_DAY, Lists.newArrayList(invested));
    
        // 获取所有兑付流水
        List<PlanAcctTitleLog> allTitleLogs = planAcctTitleLogRepo.getByPlanAndEvent(
                null, planNo, EventCodeEnum.TITLE_ALLOC, investedEnums);
        acctTitleLogMap.put(ReportTitleGroupEnum.INVESTED_ALL, Lists.newArrayList(allTitleLogs));
        return acctTitleLogMap;
    }
}