package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.model.domain.plan.PlanTaskLog;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.plan.PlanTaskLogRepo;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.general.system.common.model.IEnum;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;
/**
 * 每日十点放款池余额通知
 */

@Component
public class BalanceReportKeeper extends AbstractKeeperNode{
    @Autowired
    private PlanTaskLogRepo planTaskLogRepo;
    @Autowired
    private PlanRepo planRepo;
    @Autowired
    private AcctManager acctManager;
    @Autowired
    private WkPayManager wkPayManager;

    @Value("${wkabs.zhaolian.balanceReport.deadline:10 0 0}")
    private String deadline;
    @Value("${wkabs.org.zhaolian:1009}")
    private String zhaolianOrgCode;

    @Override
    protected Void doHandle(Date date) {

        // 判断当前时间是否大于上午十点
        Date currentTime = getCurrentStanderDate();
        Date deadlineTime = getDeadLineTime(currentTime, deadline);

        Map<String,Map<AccountAttributeEnum, Money>> buildMessageMap=new HashMap<>();
        if (currentTime.compareTo(deadlineTime) >=0) {
            LogUtil.info(LOGGER, "每日放款池余额通知-start");
            Date bizDate =getCurrentStanderDate();
            List<PlanTaskLog> tasks = planTaskLogRepo.getTasks(bizDate, PlanTaskTypeEnum.BALANCE_REPORT);
            Map<String, List<PlanTaskLog>> planTaskLogMap = tasks.stream().collect(Collectors.groupingBy(PlanTaskLog::getPlanNo));

            // 查询存续中的放款池计划
            List<PlanBase> onGoingPlans = planRepo.getPlansByStatus(PlanTypeEnum.SPV, PlanStatusEnum.ONGOING);
            for (PlanBase plan : onGoingPlans) {
                boolean isExist=false;
                if (tasks != null) {
                    isExist=planTaskLogMap.containsKey(plan.getPlanNo());
                }
                if (!isExist) {
                    Map<AccountAttributeEnum, Money> moneyMap = getMoneyMap(plan);
                    buildMessageMap.put(plan.getPlanNo(),moneyMap);
                }else{
                    continue;
                }
                PlanTaskLog planTaskLog = new PlanTaskLog();
                planTaskLog.setExecDate(bizDate);
                planTaskLog.setPlanNo(plan.getPlanNo());
                planTaskLog.setTaskType(PlanTaskTypeEnum.BALANCE_REPORT);
                planTaskLogRepo.createTask(planTaskLog);
            }
            String reportMessage = buildReportMessage(buildMessageMap,date);
            if (CollectionUtils.isNotEmpty(buildMessageMap.keySet())&&reportMessage.length() > 0) {
                // 获取用户
                Set<String> receivers = super.getAdminReceivers();
                PlanBase planBase=new PlanBase();
                super.sendBussinessNotice(planBase, zhaolianOrgCode, reportMessage, receivers);
            }

        }
        LogUtil.info(LOGGER, "每日放款池余额通知-end");

        return null;
    }

    private Map<AccountAttributeEnum,Money> getMoneyMap(PlanBase plan){

        Map<AccountAttributeEnum,Money> accountAttributeEnumMoneyMap=new HashMap<>();
        // 获取计划对应资金账户 平台虚拟户
        List<Account> accounts = acctManager.getAcctByPlanNoAndType(plan.getPlanNo(), AccountTypeEnum.PLAT_VIRTUAL, false);
        accounts.forEach(v->{
            Money bankBal = new Money();
            try {
                // 调用外部接口查询账户余额
                WkChannelNoEnum channelNo = WkChannelNoEnum.getPlatChannelNo(v.getChannelNo());
                Money balance = wkPayManager.balance(v.getActNo(), channelNo);
                if(balance!=null) {
                    bankBal.addTo(balance);
                }
                LogUtil.info(LOGGER, "{0}[{1}]查询账户余额: {2}", plan.getPlanType().getDesc(), plan.getPlanNo(), bankBal.getAmount());
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "[{0}]-ChannelNo[{1}]-ActNo[{2}]调用五矿查询账户余额异常",
                        plan.getPlanNo(), v.getChannelNo(), v.getActNo());
                return;
            }
            accountAttributeEnumMoneyMap.put(v.getActAttr(),bankBal);
        });
        return accountAttributeEnumMoneyMap;
    }
    private String buildReportMessage(Map<String,Map<AccountAttributeEnum, Money>> buildMessageMap,Date currDate){
        SimpleDateFormat sdf=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String currentTimeStr = sdf.format(currDate);
        StringBuilder sb=new StringBuilder();
        sb.append("截止到"+currentTimeStr+"\n");
        buildMessageMap.forEach((planNo,moneyMap)->{
            sb.append("放款池:"+planNo);
            moneyMap.forEach((accountType,freeMoney)->{
                switch (accountType) {
                    case RECV:{
                        sb.append("回款户余额为:"+freeMoney.getAmount());
                        break;
                    }
                    case PAY:{
                        sb.append("放款户余额为:"+freeMoney.getAmount());
                        break;
                    }
                     default:
                         throw new RuntimeException("不支持的账户类型:"+accountType);

                }
                sb.append("\n");
            });

        });
        return sb.toString();
    }
    @Override
    protected String getWatchJob() {
        return null;
    }

    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.BalanceReport;
    }
}
