/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.biz.shared.scheduler.job.keeper.node;

import com.abssqr.plat.biz.shared.scheduler.job.plan.PlanCapitalOrderJob;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Set;

/**
 * 支付任务监控
 *
 * @author xiachenxiang
 * @version com.abssqr.plat.biz.shared.scheduler.job.keeper.node: capitalOrderKeeper.java, v 0.1 2019-09-27 11:35 AM xiachenxiang Exp $
 */
@Component
public class CapitalOrderKeeper extends AbstractKeeperNode {
    @Autowired
    private PlanCapitalOrderRepo capitalOrderRepo;

    @Value("${wkabs.org.zhaolian:1009}")
    private String zhaolianOrgCode;


    @Override
    protected Void doHandle(Date date) {
        LogUtil.info(LOGGER, "支付任务监控-start");
        Long count = capitalOrderRepo.getFailCount(execTimes);
        if (count != null && count > 0) {
            String msg = MessageUtil.formatMsg("支付任务监控到有[{0}]条失败任务，请排查具体原因!", count);
            LogUtil.info(LOGGER, msg);
            // 获取用户
            Set<String> receivers = super.getAdminReceivers();
            super.sendSystemNotice("支付任务告警", zhaolianOrgCode, msg, receivers);
        }
        LogUtil.info(LOGGER, "支付任务监控-end");
        return null;
    }


    @Override
    protected IEnum getBizType() {
        return AbssqrNoticeTypeEnum.CapitalOrder;
    }

    @Override
    protected String getWatchJob() {
        return PlanCapitalOrderJob.class.getName();
    }


}
