/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import cn.hutool.json.JSONUtil;
import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.common.util.utils.DayUtil;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.acct.AcctManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.abssqr.plat.core.share.pay.PlanCapitalCreateRequest;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.Optional;

/**
 * 放款池计划 资金交易处理
 * 放款截止日 次日进行 放款户向 回款户资金划账
 *
 * @author hanfei
 * @version $Id: LenderOrgCapitalJob.java, v 0.1 2018-01-06 上午12:39 hanfei Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class LenderOrgCapitalJob extends BaseQuartzJob {

    @Autowired
    TransactionTemplate     transactionTemplate;
    @Autowired
    WkPayManager            wkPayManager;
    @Autowired
    PlanRepo                planRepo;
    @Autowired
    AcctManager             acctManager;
    @Autowired
    PlanAstManager          planAstManager;
    @Autowired
    PlanCapitalOrderRepo    planCapitalOrderRepo;
    @Autowired
    PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 每天  30 15/20 * * * ?
     */
    @Value("${sync.quartz.plan.orgcapital}")
    private String              cron;

    @Override
    protected String getCronExpr() {
        return cron;
    }

    @Override
    protected boolean isRequestRecovery() {
        return true;
    }

    @Override
    public void doExecute() {
        // 获取转让会计日
        Day curDate = orgAcctDayComponent.getTrfAcctDay();
        LogUtil.info(LOGGER, "[LnderOrgCapitalJob]-[{0}]放款计划资金交易处理-start", curDate);
        // 1. 查询存续中的放款池计划
        List<PlanBase> planBases = planRepo.getPlansByStatus(PlanTypeEnum.SPV, PlanStatusEnum.ONGOING);
        for (PlanBase planBase : planBases) {
            // 当前有资金交易单未处理完毕 则不进行账户资金处理等待下一次处理
            // 校验是否需有未完成支付单
            if (planCapitalOrderManager.repayAcctCapital(planBase.getPlanNo())){
                continue;
            }
            if (!(planBase instanceof OrganizationEntity)){
                LogUtil.error(LOGGER, "放款计划[{0}]模型不匹配", planBase);
            }
            OrganizationEntity entity = (OrganizationEntity)planBase;
            if (ToolUtil.isEmpty(entity.getStopPayDate())){
                continue;
            }
            final String planNo = entity.getPlanNo();
            // 计算当前日期与 放款截止日相差天数
            int count = DayUtil.diffDays(curDate, new Day(entity.getStopPayDate()));
            try {
                if (count >= 1) {
                    LogUtil.info(LOGGER, "放款计划[{0}]放款截止日次日资金交易处理开始", planNo);
                    // 放款截止日 次日进行 放款户向 回款户资金划账
                    this.execRemitRepo(curDate, entity);
                    LogUtil.info(LOGGER, "放款计划[{0}]放款截止日次日资金交易处理结束", planNo);
                } else {
                    // 判断放款截止日 大于当前日期 不进行资金操作
                    LogUtil.info(LOGGER, "放款计划[{0}]非放款截止日次日, 当前时间为[{1}]", planNo,curDate);
                    continue;
                }
            } catch (AbssqrBizException e) {
                if (CommonErrorCodeEnum.NO_DATA == e.getErrCode()) {
                    LogUtil.warn(LOGGER, e.getMessage());
                } else {
                    LogUtil.error(LOGGER, e, "放款计划[{0}]放款截止日次日资金交易处理异常", entity);
                }
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "放款计划[{0}]放款截止日次日资金交易处理异常", entity);
            }
        }
        LogUtil.info(LOGGER, "[LenderOrgCapitalJob]-[{0}]放款计划资金交易处理-end", curDate.getStandardDate());
    }

    /**
     * 放款户 转 回款户
     *
     * @param curDate
     * @param entity
     */
    private void execRemitRepo(Day curDate, OrganizationEntity entity) {
        // 查询放款户余额
        Money bankBal = new Money();
        // 过滤出 放款户的平台虚拟账户
        List<Account> accounts = acctManager.getAcctByPlanNoAndType(entity.getPlanNo(),
            AccountTypeEnum.PLAT_VIRTUAL,false);
        VarChecker.checkNotEmpty(accounts, "未找到对应的账号信息，[{0}]", entity.getPlanNo());
        Optional<Account> platAcctOpt = accounts.stream()
            .filter(item -> item.getActAttr() == AccountAttributeEnum.PAY)
            .findFirst();
        Account planAcct = platAcctOpt.get();
        try {
            // 3、调用外部接口查询账户余额
            WkChannelNoEnum channelNo = WkChannelNoEnum.getPlatChannelNo(planAcct.getChannelNo());
            bankBal.addTo(wkPayManager.balance(planAcct.getActNo(), channelNo));
        } catch (Throwable e) {
            LogUtil.error(LOGGER, e, "查询[{0}]五矿账户余额异常", entity.getPlanNo());
            throw e;
        }
        if (bankBal.isZero()) {
            LogUtil.info(LOGGER, "放款计划[{0}]放款户余额为0 不做操作", entity.getPlanNo());
            return;
        }
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus status) {
                // 记录资产水位表 创建资金交易订单
                planAstManager.changePlanAst(curDate.getStandardDate(), entity.getPlanNo(), (planAst, planAstLogList) -> {

                    boolean existCapital = planCapitalOrderManager.repayAcctCapital(entity.getPlanNo());
                    VarChecker.checkArgument(!existCapital,"回款户资金划账-未完成支付单，防止并发");

                    LogUtil.info(LOGGER, "放款计划[{0}]放款截止日次日进行放款户向回款户资金划账", entity.getPlanNo());
                    PlanCapitalTransTypeEnum transType = PlanCapitalTransTypeEnum.SPV_REMIT_REPO;
                    // 3、 创建支付订单
                    PlanCapitalCreateRequest request = new PlanCapitalCreateRequest();
                    request.setBizNo(SequenceUtil.genSeqNo(transType.getCode()));
                    request.setPlanNo(entity.getPlanNo());
                    request.setTargetOrgCode(entity.getPlanNo());
                    request.setTransAmt(bankBal);
                    request.setPayableAmt(bankBal);
                    request.setAccountDate(curDate.getStandardDate());
                    request.setTransType(transType);
                    request.setMemo("-放款截止日次日进行放款户向回款户资金划账");
                    PlanCapitalOrder order = planCapitalOrderManager.createCapitalOrder(request);
    
                    // 放款池放款终止转回款户 需要先增加校验金额 满足后续变更水位校验
                    planAstManager.changeCheckCashAmt(planAst, order.getTransAmt(), transType.getFlowType());
                    
                    // 更新计划资产水位表，追加在途的支出收入金额，扣减待结算支出收入金额 并登记日志
                    planAstLogList.add(planAstManager.itemChangeStage(order.getAccountDate(), planAst, TraderTypeEnum.INITIATOR,
                            StageChangeTypeEnum.ON_WAY, order.getTransAmt(), transType,
                            order.getBizNo(), JSONUtil.toJsonStr(order)));
                });
            }
        });
    }

}
