/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import com.abssqr.plat.biz.shared.handler.PlanAstStatTaskHandler;
import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.facade.enums.CommonErrorCodeEnum;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.general.system.common.util.LogUtil;
import com.general.system.tool.util.ToolUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;

/**
 * 资产清算统计业务 初始化清算汇总调度任务
 *
 * @author hanfei
 * @version $Id: PlanAstStatJob.java, v 0.1 2018-01-06 上午12:39 hanfei Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class PlanAstStatJob extends BaseQuartzJob {
    @Autowired
    private TransactionTemplate    transactionTemplate;
    @Autowired
    private PlanRepo               planRepo;
    @Autowired
    private PlanAstStatTaskHandler planAstStatTaskHandler;
    @Autowired
    private OrgAcctDayComponent    orgAcctDayComponent;
    
    /**
     * 每天 5 0/5 * * * ?
     */
    @Value("${sync.quartz.plan.stat}")
    private String                 cron;
    
    @Override
    protected String getCronExpr() {
        return cron;
    }
    
    @Override
    public void doExecute() {
        // 获取转让会计日
        Day trfDate = orgAcctDayComponent.getTrfAcctDay();
        LogUtil.info(LOGGER, "[PlanAstStatJob]-[{0}]资产清算统计业务start", trfDate.getDateString());
    
        // 1.1 查询当前 状态为 存续中 承接池计划
        List<PlanBase> onGoingPlans = planRepo.getPlansByStatus(null,PlanStatusEnum.ONGOING);
        if (ToolUtil.isEmpty(onGoingPlans)){
            return;
        }
        LogUtil.info(LOGGER, "[PlanAstStatJob]-当前存续期计划数量[{0}]", onGoingPlans.size());
        for (PlanBase entity : onGoingPlans) {
            final String planNo = entity.getPlanNo();
            try {
                LogUtil.info(LOGGER, "{0}[{1}]的统计任务初始化开始", entity.getPlanType().getDesc(), planNo);
                transactionTemplate.execute(new TransactionCallbackWithoutResult() {
                    @Override
                    protected void doInTransactionWithoutResult(TransactionStatus status) {
                        PlanBase lockedPlanBase = planRepo.lockPlanByNo(planNo);
                        if (lockedPlanBase.getStatus() != PlanStatusEnum.ONGOING) {
                            LogUtil.info(LOGGER, "计划[{0}]不在存续期，不进行统计任务", lockedPlanBase);
                            return;
                        }
    
                        planAstStatTaskHandler.handle(lockedPlanBase.getPlanNo(), trfDate);
                    }
                });
                LogUtil.info(LOGGER, "{0}[{1}]的统计任务初始化结束", entity.getPlanType().getDesc(), planNo);
                
            } catch (AbssqrBizException e) {
                if (CommonErrorCodeEnum.NO_DATA == e.getErrCode()) {
                    LogUtil.warn(LOGGER, e.getMessage());
                } else {
                    LogUtil.error(LOGGER, e, "{0}[{1}]的统计任务初始化异常", entity.getPlanType().getDesc(), planNo);
                }
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "{0}[{1}]的统计任务初始化异常", entity.getPlanType().getDesc(), planNo);
            }
        }
    }
    
    
}
