/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.biz.shared.scheduler.job.plan;

import com.abssqr.plat.biz.shared.handler.PlanFundHandler;
import com.abssqr.plat.biz.shared.scheduler.BaseQuartzJob;
import com.abssqr.plat.common.facade.enums.PlanStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.general.system.common.util.LogUtil;
import com.general.system.tool.util.ToolUtil;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionException;
import org.quartz.PersistJobDataAfterExecution;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 资金到账情况查询
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.biz.shared.scheduler.job.plan: PlanFundJob.java, v 0.1 2019-06-16 16:08 bangis.wangdf Exp $
 */
@PersistJobDataAfterExecution
@DisallowConcurrentExecution
@Component
public class PlanFundJob extends BaseQuartzJob {
    @Autowired
    private PlanFundHandler     planFundHandler;
    @Autowired
    private PlanRepo            planRepo;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    /**
     * 每天 45 11/20 * * * ?
     */
    @Value("${sync.quartz.plan.fund}")
    private String              cron;
    
    @Override
    protected String getCronExpr() {
        return cron;
    }
    
    @Override
    protected boolean isRequestRecovery() {
        return true;
    }
    
    /**
     * 计划银行账户余额查询
     * 1、放款池回款户
     * 2、受让池业务户
     *
     * @throws JobExecutionException
     */
    @Override
    public void doExecute() {
        // 获取转让会计日
        Day curDate = orgAcctDayComponent.getTrfAcctDay();
        LogUtil.info(LOGGER, "[FundJob]-[{0}]余额查询任务start", curDate);
        // 1.1 查询当前 状态为 存续中 承接池计划
        List<PlanBase> onGoingPlans = planRepo.getPlansByStatus(null,PlanStatusEnum.ONGOING);
        
        if (ToolUtil.isEmpty(onGoingPlans)){
            return;
        }
        for (PlanBase entity : onGoingPlans) {
            try {
                // 执行余额查询
                planFundHandler.executeQueryBalance(entity, curDate);
            } catch (Throwable e) {
                LogUtil.error(LOGGER, e, "[FundJob]计划[{0}]余额修正任务异常", entity);
            }
        }
    }
    
}
