/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk;

import java.util.List;

import com.abssqr.plat.common.integration.wk.domain.Result;
import com.abssqr.plat.common.integration.wk.domain.WkBody;
import com.abssqr.plat.common.integration.wk.domain.opr.OprRstHeader;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TradeList;
import com.abssqr.plat.common.integration.wk.domain.opr.response.AccountListResult;
import com.abssqr.plat.common.integration.wk.domain.opr.response.ProjectResult;
import com.abssqr.plat.common.integration.wk.domain.qry.QryRstHeader;
import com.abssqr.plat.common.integration.wk.domain.qry.response.BalanceResult;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;

/**
 * 五矿支付平台接口
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.wk: WkPayIntegration.java, v 0.1 2019-06-19 11:10 bangis.wangdf Exp $
 */
public interface WkPayIntegration {
    /**
     * 租户签约与维护
     *
     * @param seqNo 业务流水号，五矿使用UUID
     * @return
     */
    Result<OprRstHeader, ProjectResult> project(String seqNo,String projNo,String projName);

    /**
     * 账户信息维护
     *
     * @param seqNo
     * @param accountList
     * @return
     */
    Result<OprRstHeader, AccountListResult> account(String seqNo, List<WkAccount> accountList);

    /**
     * 账户余额查询 B100002
     * 最多50条
     *
     * @param seqNo
     * @param no
     * @return
     */
    Result<QryRstHeader, BalanceResult> balance(String seqNo, String no, WkChannelNoEnum channelNo);

    /**
     * 代理交易
     *
     * @param seqNo
     * @param tradeList
     * @return
     */
    Result<OprRstHeader, WkBody> trade(String seqNo, TradeList tradeList);

    /**
     * 代理交易结果查询
     *
     * @param seqNo
     * @param entryNos
     * @return
     */
    Result<QryRstHeader, TradeResult> tradeRstQry(String seqNo, List<String>  entryNos);

}
