/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk.domain.opr.request.acct;

import com.abssqr.plat.common.integration.wk.domain.WkObject;
import com.abssqr.plat.common.integration.wk.enums.WkAcctKindEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctTypeEnum;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil.Account;

import javax.xml.bind.annotation.XmlRootElement;

/**
 * 账户信息
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.wk.domain.opr.request: WkAccount.java, v 0.1 2019-06-19 18:42 bangis
 * .wangdf Exp $
 */
@XmlRootElement(name = "AccountInfo")
public class WkAccount extends WkObject {
    /**

     AccountNo	账户号	字符	32	Y	不能使用ufc(不区分大小写)开头，业务系统保证唯一并以业务系统编号开头，且只能由数字、字母和下划线组成
     AccountName	账户名称	字符	128	N	非空时使用
     TenantNo	租户编号	字符	32	Y	支付平台开通的租户编号，使用前向支付平台获取,一旦与账户绑定成功后无法修改
     CustNo	客户号/用户号	字符	64	N	不能使用ufc(不区分大小写)开头，业务系统保证唯一，且只能由数字、字母和下划线组成
     CustName	客户名称/用户名称	字符	128	N	非空时使用
     AcctType	账户类型	字符	2	Y	1-放款账户,2-扣款账户,3-商户账户,4-费用账户,5-溢缴款账户
     AcctKind	账户种类	字符	2	N	非必填,为空的情况下为 对私，指定了 账户种类 之后，按照指定的 类型进行建立。
     01为对公，02为对私

     */

    /**
     * @see Account#genAccountNo(com.abssqr.plat.common.integration.wk.enums.TradeTypeEnum, java.lang.String, com.abssqr.plat.common.facade.enums.AccountAttributeEnum)
     * 不能使用ufc(不区分大小写)开头，业务系统保证唯一并以业务系统编号开头，且只能由数字、字母和下划线组成
     * 000004_TradeTypeEnum_planNo
     */
    private String       accountNo;
    /**
     * @see Account#genAccountName(com.abssqr.plat.common.integration.wk.enums.TradeTypeEnum, java.lang.String, com.abssqr.plat.common.facade.enums.AccountAttributeEnum)
     * 不能使用ufc(不区分大小写)开头，业务系统保证唯一并以业务系统编号开头，且只能由数字、字母和下划线组成
     * 000004_扣款/放款_planName
     */
    private String         accountName;
    /**
     * 租户编号 之前生成的租户编号
     */
    private String         tenantNo;
    /**
     * @see WkSequenceUtil.Account#genCustNo(String)
     * 客户号，使用计划编号
     * 000004_planNo
     */
    private String         custNo;
    /**
     * @see WkSequenceUtil.Account#genCustName(String)
     * 客户名称 计划名称
     */
    private String         custName;
    /**
     * 账户类型	字符	2	Y	1-放款账户,2-扣款账户,3-商户账户,4-费用账户,5-溢缴款账户
     */
    private WkAcctTypeEnum acctType;
    /**
     * 账户种类	字符	2	N	非必填,为空的情况下为 对私，指定了 账户种类 之后，按照指定的 类型进行建立。
     * 01为对公，02为对私
     */
    private WkAcctKindEnum acctKind;

    public String getAccountNo() {
        return accountNo;
    }

    public void setAccountNo(String accountNo) {
        this.accountNo = accountNo;
    }

    public String getAccountName() {
        return accountName;
    }

    public void setAccountName(String accountName) {
        this.accountName = accountName;
    }

    public String getTenantNo() {
        return tenantNo;
    }

    public void setTenantNo(String tenantNo) {
        this.tenantNo = tenantNo;
    }

    public String getCustNo() {
        return custNo;
    }

    public void setCustNo(String custNo) {
        this.custNo = custNo;
    }

    public String getCustName() {
        return custName;
    }

    public void setCustName(String custName) {
        this.custName = custName;
    }

    public WkAcctTypeEnum getAcctType() {
        return acctType;
    }

    public void setAcctType(WkAcctTypeEnum acctType) {
        this.acctType = acctType;
    }

    public WkAcctKindEnum getAcctKind() {
        return acctKind;
    }

    public void setAcctKind(WkAcctKindEnum acctKind) {
        this.acctKind = acctKind;
    }
}
