/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk.impl;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;

import com.abssqr.plat.common.integration.utils.RestTemplateClient;
import com.abssqr.plat.common.integration.wk.WkPayIntegration;
import com.abssqr.plat.common.integration.wk.domain.Request;
import com.abssqr.plat.common.integration.wk.domain.Result;
import com.abssqr.plat.common.integration.wk.domain.WkBody;
import com.abssqr.plat.common.integration.wk.domain.WkHeader;
import com.abssqr.plat.common.integration.wk.domain.config.WkPayConfig;
import com.abssqr.plat.common.integration.wk.domain.opr.OprHeader;
import com.abssqr.plat.common.integration.wk.domain.opr.OprRstHeader;
import com.abssqr.plat.common.integration.wk.domain.opr.request.Project;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.AccountList;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TradeList;
import com.abssqr.plat.common.integration.wk.domain.opr.response.AccountListResult;
import com.abssqr.plat.common.integration.wk.domain.opr.response.ProjectResult;
import com.abssqr.plat.common.integration.wk.domain.qry.QryHeader;
import com.abssqr.plat.common.integration.wk.domain.qry.QryRstHeader;
import com.abssqr.plat.common.integration.wk.domain.qry.request.AccountBalanceParam;
import com.abssqr.plat.common.integration.wk.domain.qry.request.trade.TradeItem;
import com.abssqr.plat.common.integration.wk.domain.qry.request.trade.TradeQuery;
import com.abssqr.plat.common.integration.wk.domain.qry.response.BalanceResult;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.ComIdEnum;
import com.abssqr.plat.common.integration.wk.enums.LocalOrChannelEnum;
import com.abssqr.plat.common.integration.wk.enums.ProjStateEnum;
import com.abssqr.plat.common.integration.wk.enums.RequestTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.TradeQueryTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.integration.wk.utils.XMLUtils;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

/**
 * 五矿支付接口
 *
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.wk.impl: WkPayIntegrationImpl.java, v 0.1 2019-06-19 11:15 bangis
 * .wangdf Exp $
 */
@Repository
public class WkPayIntegrationImpl implements WkPayIntegration {

    private final static Logger LOGGER = CommonLoggers.WK_INTEGRATION;

    @Autowired
    private        WkPayConfig        wkPayConfig;
    @Autowired
    private        RestTemplateClient restTemplateClient;
    private static String             hostIp;
    private static String             requestStart =
        "<soapenv:Envelope xmlns:soapenv=\"http://schemas.xmlsoap.org/soap/envelope/\""
            + " xmlns:ws=\"http://ws.inf.creditapp.app/\"><soapenv:Header/><soapenv:Body><ws:doAction><!--Optional"
            + ":--><arg0>";
    private static String             requestBody  = "</arg0><!--Optional:--><arg1><![CDATA[";
    private static String             requestEnd   = "]]></arg1></ws:doAction></soapenv:Body></soapenv:Envelope>";

    static {
        try {
            InetAddress addr = InetAddress.getLocalHost();
            hostIp = addr.getHostAddress();
        } catch (UnknownHostException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 租户签约与维护
     *
     * @param seqNo
     * @return
     */
    @Override
    public Result<OprRstHeader, ProjectResult> project(String seqNo, String projNo, String projName) {
        Request<OprHeader, Project> request = new Request<>();
        OprHeader header = new OprHeader();
        buildHeader(header, seqNo, RequestTypeEnum.A100011);
        request.setHeader(header);
        Project project = new Project();
        project.setProjState(ProjStateEnum.生效);
        project.setProjName(projName);
        project.setProjNo(projNo);
        request.setBody(project);
        header.setTradeCount(1);

        return doPost(request, new Result(OprRstHeader.class,
            ProjectResult.class));
    }

    /**
     * 账户信息维护
     *
     * @param seqNo
     * @param accountList
     * @return
     */
    @Override
    public Result<OprRstHeader, AccountListResult> account(String seqNo, List<WkAccount> accountList) {
        VarChecker.checkNotEmpty(accountList);
        // 组装请求 header
        OprHeader header = new OprHeader();
        buildHeader(header, seqNo, RequestTypeEnum.A100002);
        header.setTradeCount(accountList.size());
        // 组装请求 body
        AccountList body = new AccountList();
        body.setAccountList(accountList);
        // 组装参数
        Request<OprHeader, AccountList> request = new Request<>();
        request.setHeader(header);
        request.setBody(body);
        return doPost(request, new Result(OprRstHeader.class,
            AccountListResult.class));
    }

    /**
     * 账户余额查询 B100002
     * 最多50条
     *
     * @param seqNo
     * @param no
     * @return
     */
    @Override
    public Result<QryRstHeader, BalanceResult> balance(String seqNo, String no, WkChannelNoEnum channelNo) {
        VarChecker.checkNotEmpty(no);
        Request<QryHeader, AccountBalanceParam> request = new Request();
        QryHeader header = new QryHeader();
        header.setPageNo(1);
        header.setPageSize(100);
        buildHeader(header, seqNo, RequestTypeEnum.B100002);
        request.setHeader(header);
        AccountBalanceParam body = new AccountBalanceParam();
        body.setNo(no);
        body.setChannelNo(channelNo);
        body.setLocalOrChannel(LocalOrChannelEnum.getByCode(wkPayConfig.getBalanceType()));
        request.setBody(body);
        return doPost(request, new Result(QryRstHeader.class,
            BalanceResult.class));
    }

    /**
     * 交易接口调用
     *
     * @param seqNo
     * @param tradeList
     * @return
     */
    @Override
    public Result<OprRstHeader, WkBody> trade(String seqNo, TradeList tradeList) {
        VarChecker.checkNotNull(tradeList);
        Request<OprHeader, TradeList> request = new Request<>();
        OprHeader header = new OprHeader();
        buildHeader(header, seqNo, RequestTypeEnum.P100001);
        header.setTradeCount(tradeList.getTradeList().size());
        header.setReturnUrl(wkPayConfig.getTradeCallBackUrl());
        request.setHeader(header);
        request.setBody(tradeList);
        return doPost(request, new Result(OprRstHeader.class,
            AccountListResult.class));
    }

    /**
     * 交易结果查询
     *
     * @param seqNo
     * @param entryNos
     * @return
     */
    @Override
    public Result<QryRstHeader, TradeResult> tradeRstQry(String seqNo, List<String> entryNos) {
        VarChecker.checkNotEmpty(entryNos);
        Request<QryHeader, TradeQuery> request = new Request<>();
        QryHeader header = new QryHeader();
        buildHeader(header, seqNo, RequestTypeEnum.B100007);
        header.setPageNo(1);
        header.setPageSize(entryNos.size());
        request.setHeader(header);
        TradeQuery body = new TradeQuery();
        body.setQueryType(TradeQueryTypeEnum.条目号);
        List<TradeItem> list = Lists.newArrayList();
        entryNos.forEach(entryNo -> {
            TradeItem item = new TradeItem();
            item.setNo(entryNo);
            list.add(item);
        });
        body.setQueryList(list);
        request.setBody(body);
        return doPost(request, new Result(QryRstHeader.class, TradeResult.class));
    }

    private <T extends Result> T doPost(Request request, T result) {
        String requestXml = genFullXml(request);
        LogUtil.info(LOGGER, "**********[request.header.type]-{0}", request.getHeader().getRequestType());
        LogUtil.info(LOGGER, "**********[request.body]-{0}", requestXml);
        String response = "";
        try {
            response = restTemplateClient.postXml(wkPayConfig.getHost(), requestXml);
        } finally {
            VarChecker.checkNotEmpty(response,"response is null");
            LogUtil.info(LOGGER, "**********[response]-{0}", response);
        }

        return toModel(response, result);
    }

    private <T extends Result> T toModel(String resultXml, T result) {
        int start = StringUtils.indexOf(resultXml, "<return>");
        int end = StringUtils.indexOf(resultXml, "</return>");
        String substring = StringUtils.substring(resultXml, start + 8, end);
        String aResult = StringUtils.replaceAll(substring, "&lt;", "<");
        aResult = StringUtils.replaceAll(aResult, "&gt;", ">");
        aResult = StringUtils.replaceAll(aResult, "&quot;", "\"");
        LogUtil.info(LOGGER, "**********[request.body]-result-{0}", aResult);
        XMLUtils.xmlToModel(aResult, result, true);
        return result;
    }

    private void buildHeader(WkHeader wkHeader, String seqNo, RequestTypeEnum requestType) {
        wkHeader.setComId(ComIdEnum.资产转让);
        wkHeader.setAsyn(requestType.getAsyn());
        wkHeader.setRequestType(requestType);
        wkHeader.setSendTime(SystemDateUtil.getSystemDate());
        wkHeader.setComIP(hostIp);
        wkHeader.setUUID(seqNo);
    }

    private String genFullXml(Request request) {
        StringBuilder stringBuilder = new StringBuilder(requestStart);
        stringBuilder.append(request.getHeader().getRequestType().getCode());
        stringBuilder.append(requestBody);
        stringBuilder.append(XMLUtils.modelToXml(request, true));
        stringBuilder.append(requestEnd);
        return stringBuilder.toString();
    }
}
