/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk.utils;

import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.integration.wk.enums.ComIdEnum;
import com.abssqr.plat.common.integration.wk.enums.EnTradeTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.TradeTypeEnum;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.general.system.common.util.StringFormatUtil;

import static com.general.system.common.util.MessageUtil.formatMsg;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.wk.utils: WkSequenceUtil.java, v 0.1 2019-06-21 22:33 bangis.wangdf Exp $
 */
public class WkSequenceUtil {
    public static class Account {

        /**
         * 生成五矿需要的虚拟账户号
         *
         * @param tradeType
         * @param planNo
         * @return
         */
        public static String genAccountNo(TradeTypeEnum tradeType, String planNo, AccountAttributeEnum actAttr) {

            return formatMsg("{0}_{1}_{2}_{3}",
                //前缀
                ComIdEnum.资产转让.getCode(), tradeType.getCode(), planNo,actAttr.getCode());
        }

        /**
         * 生成五矿需要的虚拟账户号
         *
         * @param tradeType
         * @param planName
         * @return
         */
        public static String genAccountName(TradeTypeEnum tradeType, String planName, AccountAttributeEnum actAttr) {

            return formatMsg("{0}_{1}_{2}-{3}-{4}",
                //前缀
                ComIdEnum.资产转让.getCode(), tradeType.getCode(), planName, tradeType.getDesc(),actAttr.getDesc());
        }

        /**
         * 生成五矿需要的虚拟账户号
         *
         * @param planNo
         * @return
         */
        public static String genCustNo(String planNo) {

            return formatMsg("{0}_{1}",
                //前缀
                ComIdEnum.资产转让.getCode(), planNo);
        }

        /**
         * 生成五矿需要的虚拟账户号
         *
         * @param planName
         * @return
         */
        public static String genCustName(String planName) {

            return formatMsg("{0}_{1}",
                //前缀
                ComIdEnum.资产转让.getCode(), planName);
        }

    }

    public static class Trade {
        /**
         * 交易号
         *
         * @return
         */
        public static String genTradeNo() {
            return formatMsg("{0}_{1}", ComIdEnum.资产转让.getCode(), SequenceUtil.genWkSeqNo("td"));
        }

        /**
         * 条目号
         *
         * @return
         */
        public static String genEntryNo() {
            return formatMsg("{0}_{1}", ComIdEnum.资产转让.getCode(), SequenceUtil.genWkSeqNo("en"));
        }

        /**
         * 条目类型编号
         *
         * @return
         */
        public static String genEntryTypeNo(EnTradeTypeEnum enTradeType, int idx) {
            return formatMsg("{0}{1}", enTradeType.getCode(),
                StringFormatUtil.formatTransSeqByNumber(Long.valueOf(idx)));
        }
    }
}
