/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.common.integration.wk.utils.converter;

import java.lang.reflect.Field;
import java.util.Date;
import java.util.List;
import java.util.Map;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;

import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.XmlUtil;
import com.abssqr.plat.common.integration.wk.domain.WkBody;
import com.abssqr.plat.common.integration.wk.domain.WkObject;
import com.abssqr.plat.common.integration.wk.enums.WkEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.exception.AbssqrInvalidParamException;
import com.general.system.common.model.IEnum;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.IEnumUtil;
import com.general.system.common.util.LogUtil;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

/**
 * XML节点转换
 * @author bangis.wangdf
 * @version com.abssqr.plat.common.integration.wk.utils.converter: ElementConverters.java, v 0.1 2019-06-18 17:26 bangis
 * .wangdf Exp $
 */
public class ElementConverters {
    /**
     * Dispatcher调度日志
     */
    public final static Logger LOGGER = LoggerFactory.getLogger(ElementConverters.class);
    
    public static void appendElement(Element element, Object model, boolean uncapFirst) {
        if (model == null) {
            element.setTextContent(null);
            return;
        }

        if (model instanceof Integer) {
            element.setTextContent(model.toString());
        } else if (model instanceof Long) {
            element.setTextContent(model.toString());
        } else if (model instanceof Double) {
            element.setTextContent(model.toString());
        } else if (model instanceof String) {
            element.setTextContent(model.toString());
        } else if (model instanceof IEnum) {
            element.setTextContent(String.valueOf(((IEnum)model).getCode()));
        } else if (model instanceof Date) {
            element.setTextContent(DateTimeUtil.formatYMDHMSSLong((Date)model));
        } else if (model instanceof List) {
            for (Object obj : ((List)model)) {
                if (obj instanceof WkObject) {
                    String elemntName = elementName((WkObject)obj, uncapFirst);
                    Element childElement = appendElement(element, elemntName);
                    appendElement(childElement, obj, uncapFirst);
                } else {
                    //不支持的类型
                    LogUtil.error(LOGGER,"转五矿支付XML时仅支持WkObject");
                    throw new AbssqrInvalidParamException("转五矿支付XML时仅支持WkObject");
                }
            }
        } else if (model instanceof WkBody) {
            String elemntName = elementAnnotationName((WkObject)model, uncapFirst);
            Element childElement = element;
            if (StringUtils.isNotBlank(elemntName)) {
                childElement = appendElement(element, elemntName);
            }
            Field[] fields = ReflectUtil.getFields(model.getClass());
            if (fields != null) {
                for (Field field : fields) {
                    String childElemntName = elementName(field, uncapFirst);
                    Element _childElement = appendElement(childElement, childElemntName);
                    Object fieldValue = ReflectUtil.getFieldValue(model, field);
                    appendElement(_childElement, fieldValue, uncapFirst);
                }
            }
        } else if (model instanceof WkObject) {
            Field[] fields = ReflectUtil.getFields(model.getClass());
            if (fields != null) {
                for (Field field : fields) {
                    String elemntName = elementName(field, uncapFirst);
                    Element childElement = appendElement(element, elemntName);
                    Object fieldValue = ReflectUtil.getFieldValue(model, field);
                    appendElement(childElement, fieldValue, uncapFirst);
                }
            }
        } else {
            //不支持的类型
            LogUtil.error(LOGGER,"转五矿支付XML时不支持的类型："+model.getClass());
            throw new AbssqrInvalidParamException("转五矿支付XML时不支持的类型："+model.getClass());
        }

    }

    public static String fileName(boolean uncapFirst, String name) {
        return uncapFirst ? StrUtil.upperFirst(name) : StrUtil.lowerFirst(
            name);
    }
    
    public static Element appendElement(Element parent, String tagName) {
        Element child = parent.getOwnerDocument().createElement(tagName);
        parent.appendChild(child);
        return child;
    }

    public static <T> void elementFillModel(Element element, T model, Field field, Class fieldClazz,
                                            boolean uncapFirst) {
        if (element == null) {
            return;
        }
        Class<?> typeClass = field.getType();
        if (String.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (Integer.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (Long.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (Double.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (IEnum.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (Date.class.isAssignableFrom(typeClass)) {
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        } else if (List.class.isAssignableFrom(typeClass)) {
            List list = Lists.newArrayList();
            Class aClass = listFClass(field);
            if (WkObject.class.isAssignableFrom(aClass)) {
                try {
                    Object instance = aClass.newInstance();
                    String childFileName = elementName((WkObject)instance, uncapFirst);
                    Field[] fields = ReflectUtil.getFields(aClass);
                    List<Element> elements = XmlUtil.getElements(element, childFileName);
                    if (CollectionUtils.isNotEmpty(elements)) {
                        for (Element _element : elements) {
                            instance = aClass.newInstance();
                            for (Field childField : fields) {
                                String _childFileName = fileName(uncapFirst, childField.getName());
                                Element childElement = XmlUtil.getElement(_element, _childFileName);
                                elementFillModel(childElement, instance, childField, childField.getType(), uncapFirst);
                            }
                            list.add(instance);
                        }
                    }
                } catch (Throwable e) {
                    LogUtil.error(LOGGER,"五矿XML转内部模型失败");
                    throw new AbssqrBizException("五矿XML转内部模型失败",e);
                }

            } else {
                LogUtil.error(LOGGER,"五矿XML转内部模型时不支持的类型："+model.getClass());
                throw new AbssqrInvalidParamException("五矿XML转内部模型时不支持的类型："+model.getClass());
            }
            ReflectUtil.setFieldValue(model, field, list);
        } else if (WkBody.class.isAssignableFrom(typeClass)) {
            try {
                Object childObj = fieldClazz.newInstance();
                Element _element = element;
                String elementName = elementAnnotationName((WkObject)childObj, uncapFirst);
                if (StringUtils.isNotBlank(elementName)) {
                    _element = XmlUtil.getElement(element, elementName);
                }
                Field[] fields = ReflectUtil.getFields(childObj.getClass());
                for (Field childField : fields) {
                    String childFileName = fileName(uncapFirst, childField.getName());
                    Element childElement = XmlUtil.getElement(_element, childFileName);
                    elementFillModel(childElement, childObj, childField, childField.getType(), uncapFirst);
                }
                ReflectUtil.setFieldValue(model, field, childObj);
            } catch (Throwable e) {
                throw new RuntimeException(e);
            }
        } else if (WkObject.class.isAssignableFrom(typeClass)) {
            try {
                Object childObj = fieldClazz.newInstance();

                Field[] fields = ReflectUtil.getFields(childObj.getClass());
                for (Field childField : fields) {
                    String childFileName = fileName(uncapFirst, childField.getName());
                    Element childElement = XmlUtil.getElement(element, childFileName);
                    elementFillModel(childElement, childObj, childField, childField.getType(), uncapFirst);
                }
                ReflectUtil.setFieldValue(model, field, childObj);
            } catch (Throwable e) {
                throw new RuntimeException(e);
            }
        }else{
            ReflectUtil.setFieldValue(model, field, elementText(element, typeClass));
        }
    }

    private static Map<String, Class> listClassMap = Maps.newHashMap();

    private static Class listFClass(Field field) {
        String typeName = field.getGenericType().getTypeName();
        Class result = listClassMap.get(typeName);
        if (result == null) {
            String className = StringUtils.substring(typeName, StringUtils.indexOf(typeName, "<") + 1,
                StringUtils.indexOf(typeName, ">"));
            try {
                result = Class.forName(className);
                listClassMap.put(typeName, result);
            } catch (ClassNotFoundException e) {
            }
        }
        return result;
    }

    private static Object elementText(Element element, Class clazz) {
        String text = element == null ? null : element.getTextContent();
        Object result = null;
        if (StringUtils.isNotBlank(text)) {
            if (String.class.isAssignableFrom(clazz)) {
                result = text;
            } else if (Integer.class.isAssignableFrom(clazz)) {
                result = Integer.valueOf(text);
            }else if (int.class.isAssignableFrom(clazz)) {
                result = Integer.valueOf(text);
            } else if (Long.class.isAssignableFrom(clazz)) {
                result = Long.valueOf(text);
            }else if (long.class.isAssignableFrom(clazz)) {
                String cent = StringUtils.removeEnd(text,".00");
                result = Long.valueOf(cent);
            } else if (Double.class.isAssignableFrom(clazz)) {
                result = Double.valueOf(text);
            }else if (double.class.isAssignableFrom(clazz)) {
                result = Double.valueOf(text);
            } else if (WkEnum.class.isAssignableFrom(clazz)) {
                List enumList = IEnumUtil.getEnumList(clazz);
                if (CollectionUtils.isNotEmpty(enumList)) {
                    result = ((WkEnum)enumList.get(0)).getByWkCode(text);
                }
            } else if (IEnum.class.isAssignableFrom(clazz)) {
                result = IEnumUtil.getEnum(clazz, text);
            } else if (Date.class.isAssignableFrom(clazz)) {
                result = DateTimeUtil.parseInTolerance(text);
            } else {
                LogUtil.error(LOGGER,"五矿XML转内部模型失败");
                throw new AbssqrBizException("五矿XML转内部模型失败");
            }
        }
        return result;
    }

    public static String elementName(Field field, boolean uncapFirst) {
        String elemntName = ElementConverters.fileName(uncapFirst, field.getName());
        XmlElement annotation = field.getAnnotation(XmlElement.class);
        if (annotation != null) {
            elemntName = annotation.name();
        }
        return elemntName;
    }

    public static String elementName(WkObject wkObject, boolean uncapFirst) {
        String rootStr = elementAnnotationName(wkObject, uncapFirst);
        if (StringUtils.isBlank(rootStr)) {
            rootStr = uncapFirst ? wkObject.getClass().getSimpleName() : StrUtil.lowerFirst(
                wkObject.getClass().getSimpleName());
        }
        return rootStr;
    }
    
    private static String elementAnnotationName(WkObject wkObject, boolean uncapFirst) {
        String rootStr = null;
        XmlRootElement xmlRootElement = wkObject.getClass().getAnnotation(XmlRootElement.class);
        XmlElement xmlElement = wkObject.getClass().getAnnotation(XmlElement.class);
        if (xmlElement != null) {
            rootStr = xmlElement.name();
        } else if (xmlRootElement != null) {
            rootStr = xmlRootElement.name();
        }
        return rootStr;
    }

}
