/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.wk.pay;

import java.util.List;
import java.util.Map;

import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil.Trade;
import com.general.system.common.model.Money;
import com.google.common.collect.Maps;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.wk.pay: WkPayService.java, v 0.1 2019-12-15 14:22 bangis.wangdf Exp $
 */
public interface WkPayService {
    Map<String, WkPayService> serviceMap = Maps.newHashMap();

    /**
     * 租户签约与维护
     *
     * @return
     */
    boolean project();

    /**
     * 创建账户号
     * 重复调用不会改天
     *
     * @return 返回支付平台虚拟号
     */
    WkAccount createAccount(String planNo, String planName, AccountAttributeEnum actAttr);

    /**
     * 查询账户余额
     * 创建完虚拟号后再没有关联实际银行卡时result.getBody()=null
     *
     * @param accountNo
     * @return
     */
    Money balance(String accountNo, WkChannelNoEnum channelNo);

    /**
     * 只有放款逻辑
     * 五矿支付接口返回状态只有受理成功与平台异常，
     * 平台异常时无法保证资金是否会转让
     *
     * @param sourceAcct 源账号
     * @param targetAcct 目标账号
     * @param money
     * @param remark1 备注
     * @return 条目号
     * @see Trade#genTradeNo()
     */
    String trade(Account sourceAcct, Account targetAcct, Money money,String remark1);

    /**
     * 代理交易结果查询
     * 只支持4 做查询
     *
     * @param entryNos
     * @return TradeResult 可能为空
     */
    TradeResult tradeRstQry(List<String> entryNos);

    default String type() {
        return "mock";
    }

    default WkPayService getInstance(String type){
        WkPayService wkPayService = serviceMap.get(type);
        if(wkPayService==null){
            wkPayService = serviceMap.get("mock");
        }
        return wkPayService;
    }

    default void register() {
        serviceMap.put(type(), this);
    }
}
