/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.wk.pay.impl;

import java.util.List;

import com.abssqr.plat.common.dal.mysql.auto.dao.LoanDAO;
import com.abssqr.plat.common.dal.mysql.auto.resultmap.LoanPropCntStat;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.PlanTypeEnum;
import com.abssqr.plat.common.facade.enums.TaskStatusEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.facade.model.plan.PlanAst;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.integration.wk.domain.config.WkPayConfig;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.TradeTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctKindEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil;
import com.abssqr.plat.common.model.repo.acct.AcctRepo;
import com.abssqr.plat.common.model.repo.ast.LoanRepository;
import com.abssqr.plat.common.model.repo.plan.PlanAstRepository;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.general.system.common.model.Money;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.wk.pay.impl: WkPayMockServiceImpl.java, v 0.1 2019-12-15 14:40 bangis.wangdf
 * Exp $
 */
@Service
public class WkPayMockServiceImpl extends AbsWkPayService {
    @Autowired
    private WkPayConfig          wkPayConfig;
    @Autowired
    private PlanCapitalOrderRepo planCapitalOrderRepo;
    @Autowired
    private PlanAstRepository    planAstRepository;
    @Autowired
    private AcctRepo             acctRepo;
    @Autowired
    private PlanManager          planManager;
    @Autowired
    private LoanDAO              loanDAO;

    @Override
    public boolean project() {
        return true;
    }

    @Override
    public WkAccount createAccount(String planNo, String planName, AccountAttributeEnum actAttr) {
        WkAccount account = new WkAccount();
        account.setAccountName(WkSequenceUtil.Account.genAccountName(TradeTypeEnum.放款, planName, actAttr));
        account.setAccountNo(WkSequenceUtil.Account.genAccountNo(TradeTypeEnum.放款, planNo, actAttr));
        account.setAcctKind(WkAcctKindEnum.对公);
        account.setAcctType(WkAcctTypeEnum.放款账户);
        account.setCustName(WkSequenceUtil.Account.genCustName(planName));
        account.setCustNo(WkSequenceUtil.Account.genCustNo(planNo));
        // 租户编号
        account.setTenantNo(wkPayConfig.getProjNo());
        return account;
    }

    @Override
    public Money balance(String accountNo, WkChannelNoEnum channelNo) {
        Account account = acctRepo.getAcctsByAccountNo(accountNo);
        VarChecker.checkNotNull(account);
        PlanAst planAst = planAstRepository.getByPlanNo(account.getPlanNo());
        PlanBase planBase = planManager.getPlanFullByNo(account.getPlanNo());
        if (planBase.getPlanType() == PlanTypeEnum.SPV) {
            //回款户
            if (account.getActAttr() == AccountAttributeEnum.RECV) {
                //受让池 -- 非初始化查询
                if (planAst.getStatus() != TaskStatusEnum.INIT) {
                    return planAst.getCash();
                } else {
                    return planBase.getTotalAmt();
                }
            } else if (account.getActAttr() == AccountAttributeEnum.PAY) {

                LoanPropCntStat loanPropCntStat = loanDAO.statPlanBal(account.getPlanNo());
                if (loanPropCntStat != null) {
                    return loanPropCntStat.getAmt().add(new Money("10000"));
                }
            }
        } else {
            //受让池 -- 非初始化查询
            if (planAst.getStatus() != TaskStatusEnum.INIT) {
                return planAst.getCash();
            } else {
                //初始化查询
                return planBase.getTotalAmt();
            }
        }
        return new Money(0);
    }

    @Override
    public String trade(Account sourceAcct, Account targetAcct, Money money, String remark1) {
        //条目组装
        String enNo = WkSequenceUtil.Trade.genEntryNo();
        return "mc" + enNo;
    }

    @Override
    public TradeResult tradeRstQry(List<String> entryNos) {
        TradeResult result = new TradeResult();
        result.setPayInfoList(
                TradeResult.mockDate(planCapitalOrderRepo.queryOrderListByOutAcptNo(TaskStatusEnum.WAITING, entryNos)));
        return result;
    }

    @Override
    public String type() {
        return "mock";
    }
}
