/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.core.service.wk.pay.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.common.facade.enums.AbssqrNoticeTypeEnum;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.enums.AccountTypeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.integration.wk.WkPayIntegration;
import com.abssqr.plat.common.integration.wk.domain.Result;
import com.abssqr.plat.common.integration.wk.domain.WkBody;
import com.abssqr.plat.common.integration.wk.domain.config.WkPayConfig;
import com.abssqr.plat.common.integration.wk.domain.opr.OprRstHeader;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TdAccountInfo;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TdEntryInfo;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TdSourceAccountInfo;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TdTargetAccountInfo;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TradeInfo;
import com.abssqr.plat.common.integration.wk.domain.opr.request.trade.TradeList;
import com.abssqr.plat.common.integration.wk.domain.opr.response.AccountListResult;
import com.abssqr.plat.common.integration.wk.domain.opr.response.ProjectResult;
import com.abssqr.plat.common.integration.wk.domain.qry.QryRstHeader;
import com.abssqr.plat.common.integration.wk.domain.qry.response.BalanceResult;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.ComIdEnum;
import com.abssqr.plat.common.integration.wk.enums.EnTradeTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.PlatResultCodeEnum;
import com.abssqr.plat.common.integration.wk.enums.TradeTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAccountTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctKindEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctStatusEnum;
import com.abssqr.plat.common.integration.wk.enums.WkAcctTypeEnum;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.integration.wk.enums.WkResultCodeEnum;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil;
import com.abssqr.plat.common.integration.wk.utils.WkSequenceUtil.Trade;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.seq.SequenceUtil;
import com.abssqr.plat.common.util.loggers.CommonLoggers;
import com.abssqr.plat.core.service.notice.AbssqrNoticeManager;
import com.general.system.common.model.Money;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Set;

/**
 * @author bangis.wangdf
 * @version com.abssqr.plat.core.service.wk.pay: WkPayOnLineServiceImpl.java, v 0.1 2019-12-15 14:38 bangis.wangdf Exp $
 */
@Service
public class WkPayOnLineServiceImpl extends AbsWkPayService {
    private final static Logger LOGGER = CommonLoggers.WK_INTEGRATION;

    @Autowired
    private WkPayConfig          wkPayConfig;
    @Autowired
    private WkPayIntegration     wkPayIntegration;
    @Autowired
    private AbssqrNoticeManager  abssqrNoticeManager;

    
    @Override
    public String type() {
        return "prod";
    }

    /**
     * 租户签约与维护
     *
     * @return
     */
    @Override
    public boolean project() {
        Result<OprRstHeader, ProjectResult> result = wkPayIntegration.project(SequenceUtil.genWkSeqNo(),
            wkPayConfig.getProjNo(), wkPayConfig.getProjName());
        try {
            VarChecker.checkNotNull(result, "创建支付平台租约号[{0}-{1}]失败-result=null", wkPayConfig.getProjNo(),
                wkPayConfig.getProjName());
            VarChecker.checkNotNull(result.getHeader(), "创建支付平台租约号[{0}-{1}]失败-result.getHeader=null",
                wkPayConfig.getProjNo(), wkPayConfig.getProjName());
            VarChecker.checkNotNull(result.getBody(), "创建支付平台租约号[{0}-{1}]失败-result.getBody=null",
                wkPayConfig.getProjNo(),
                wkPayConfig.getProjName());
        } catch (BaseCommonException e) {
            this.sendNotice(e.getMessage());
            throw e;
        }
        return result.getHeader().getResultCode() == PlatResultCodeEnum.成功
            && result.getBody().getResultCode() == WkResultCodeEnum.成功;
    }

    /**
     * 创建账户号
     * 重复调用不会改天
     *
     * @return 返回支付平台虚拟号
     */
    @Override
    public WkAccount createAccount(String planNo, String planName, AccountAttributeEnum actAttr) {
        WkAccount account = new WkAccount();
        account.setAccountName(WkSequenceUtil.Account.genAccountName(TradeTypeEnum.放款, planName, actAttr));
        account.setAccountNo(WkSequenceUtil.Account.genAccountNo(TradeTypeEnum.放款, planNo, actAttr));
        account.setAcctKind(WkAcctKindEnum.对公);
        account.setAcctType(WkAcctTypeEnum.放款账户);
        account.setCustName(WkSequenceUtil.Account.genCustName(planName));
        account.setCustNo(WkSequenceUtil.Account.genCustNo(planNo));
        // 租户编号
        account.setTenantNo(wkPayConfig.getProjNo());
        List<WkAccount> accountList = CollectionUtil.newArrayList(account);
        Result<OprRstHeader, AccountListResult> result = wkPayIntegration.account(SequenceUtil.genWkSeqNo(),
            accountList);
        try {
            VarChecker.checkNotNull(result, "创建支付平台创建账户号[{0}}]失败-result=null", account);
            VarChecker.checkNotNull(result.getHeader(), "创建支付平台创建账户号[{0}}]失败-result.getHeader()=null", account);
            VarChecker.checkNotNull(result.getBody(), "创建支付平台创建账户号[{0}}]失败-result.getBody()=null", account);

            VarChecker.checkEquals(result.getHeader().getResultCode(), PlatResultCodeEnum.成功,
                "创建支付平台创建账户号[{0}}]失败-result.getHeader.result!=成功", account);

            VarChecker.checkArgument(CollectionUtils.isNotEmpty(result.getBody().getAccountList()),
                "创建支付平台创建账户号[{0}}]失败-result.getBody().getAccountList()()=null", account);
            VarChecker.checkEquals(result.getBody().getAccountList().get(0).getStatus(), WkAcctStatusEnum.成功);
        } catch (BaseCommonException e) {
            this.sendNotice(e.getMessage());
            throw e;
        }
        return account;
    }

    /**
     * 查询账户余额
     * 创建完虚拟号后再没有关联实际银行卡时result.getBody()=null
     *
     * @param accountNo
     * @return
     */
    @Override
    public Money balance(String accountNo, WkChannelNoEnum channelNo) {
        Result<QryRstHeader, BalanceResult> result = wkPayIntegration.balance(SequenceUtil.genWkSeqNo(), accountNo,
            channelNo);
        try {
            VarChecker.checkNotNull(result, "查询账户余额[{0}}]失败-result=null", accountNo);
            VarChecker.checkNotNull(result.getHeader(), "查询账户余额[{0}}]失败-result.getHeader()=null", accountNo);
            VarChecker.checkNotNull(result.getBody(), "查询账户余额[{0}}]失败-result.getBody()=null", accountNo);
            VarChecker.checkEquals(result.getHeader().getResultCode(), PlatResultCodeEnum.成功,
                    "查询账户余额[{0}}]失败-result.getHeader.result!=10000;{1}",
                    accountNo, result.getHeader().getResultMsg());

            VarChecker.checkArgument(CollectionUtils.isNotEmpty(result.getBody().getBalanceList()),
                "查询账户余额[{0}}]失败-没有查到余额",
                accountNo);
        } catch (BaseCommonException e) {
            this.sendNotice(e.getMessage());
            throw e;
        }
        return Money.createWithCent(result.getBody().getBalanceList().get(0).getBalance());
    }

    /**
     * 只有放款逻辑
     * 五矿支付接口返回状态只有受理成功与平台异常，
     * 平台异常时无法保证资金是否会转让
     *
     * @param sourceAcct 源账号
     * @param targetAcct 目标账号
     * @param money
     * @return 条目号
     * @see Trade#genTradeNo()
     */
    @Override
    public String trade(Account sourceAcct, Account targetAcct, Money money, String remark1) {
        VarChecker.checkEquals(sourceAcct.getActType(), AccountTypeEnum.PLAT_VIRTUAL);
        TradeList tradeList = new TradeList();
        List<TradeInfo> list = Lists.newArrayList();
        TradeInfo tradeInfo = new TradeInfo();
        // 受理单号
        String tradeNo = SequenceUtil.genWkSeqNo(ComIdEnum.资产转让.getCode());
        tradeInfo.setTradeNo(tradeNo);
        tradeInfo.setTotalEntryNum(1L);
        tradeInfo.setTradeTypeNo(TradeTypeEnum.放款);

        //条目组装
        String enNo = WkSequenceUtil.Trade.genEntryNo();
        List<TdEntryInfo> entryList = Lists.newArrayList();
        TdEntryInfo tdEntryInfo = new TdEntryInfo();
        tdEntryInfo.setRemark1(remark1);
        tdEntryInfo.setAmount(money.getCent());
        //SourceTarget为平台账户时不能填
        if (sourceAcct.getActType() == AccountTypeEnum.BANK) {
            tdEntryInfo.setChannelNo(WkChannelNoEnum.getByCode(sourceAcct.getChannelNo()));
        }
        tdEntryInfo.setEntryNo(enNo);
        tdEntryInfo.setEntryTypeNo(WkSequenceUtil.Trade.genEntryTypeNo(EnTradeTypeEnum.放款, 1));
        tdEntryInfo.setEnTradeType(EnTradeTypeEnum.放款);
        //账号组装
        List<TdSourceAccountInfo> sourceAccountList = Lists.newArrayList();
        TdSourceAccountInfo tdSourceAccountInfo = new TdSourceAccountInfo();
        tdSourceAccountInfo.setLevel(1);
        tdSourceAccountInfo.setAccountInfo(builderAccountInfo(sourceAcct));
        sourceAccountList.add(tdSourceAccountInfo);
        tdEntryInfo.setSourceAccountList(sourceAccountList);
        List<TdTargetAccountInfo> targetAccountList = Lists.newArrayList();
        TdTargetAccountInfo tdTargetAccountInfo = new TdTargetAccountInfo();
        tdTargetAccountInfo.setLevel(1);
        tdTargetAccountInfo.setAccountInfo(builderAccountInfo(targetAcct));
        targetAccountList.add(tdTargetAccountInfo);
        tdEntryInfo.setTargetAccountList(targetAccountList);
        entryList.add(tdEntryInfo);
        // 设置入账条目
        tradeInfo.setEntryList(entryList);
        // 组装交易信息
        list.add(tradeInfo);
        // 组装交易信息集合
        tradeList.setTradeList(list);

        try {
            Result<OprRstHeader, WkBody> trade = wkPayIntegration.trade(SequenceUtil.genWkSeqNo(), tradeList);
            if (trade != null) {
                LogUtil.info(LOGGER, "五矿代理交易 - [{0}-{1}-{2}]", SequenceUtil.genWkSeqNo(), enNo,
                    trade.getHeader().getResultCode());
            } else {
                String msg = MessageUtil.formatMsg("五矿代理交易返回结果异常 - [{0}-{1}]", SequenceUtil.genWkSeqNo(), enNo);
                LogUtil.warn(LOGGER, msg);
                sendNotice(msg);
            }
        } catch (AbssqrBizException e) {
            LogUtil.error(LOGGER, e, "ABS系统-内部异常 - [{0}-{1}]", SequenceUtil.genWkSeqNo(), enNo);
            throw e;
        } catch (Throwable e) {
            LogUtil.error(LOGGER, e, "五矿代理交易返回网络等异常 - [{0}-{1}]", SequenceUtil.genWkSeqNo(), enNo);
        }
        return enNo;
    }

    /**
     * 代理交易结果查询
     * 只支持4 做查询
     *
     * @param entryNos
     * @return TradeResult 可能为空
     */
    @Override
    public TradeResult tradeRstQry(List<String> entryNos) {
        Result<QryRstHeader, TradeResult> result = wkPayIntegration.tradeRstQry(SequenceUtil.genWkSeqNo(), entryNos);
        try {
            VarChecker.checkNotNull(result, "代理交易结果查询[{0}}]失败-result=null", entryNos);
            VarChecker.checkNotNull(result.getHeader(), "代理交易结果查询[{0}}]失败-result.getHeader()=null", entryNos);
            VarChecker.checkNotNull(result.getBody(), "代理交易结果查询[{0}}]失败-result.getBody()=null", entryNos);
        } catch (BaseCommonException e) {
            this.sendNotice(e.getMessage());
            throw e;
        }
        return result.getBody();
    }

    private TdAccountInfo builderAccountInfo(Account account) {
        TdAccountInfo tdSourceAccount = new TdAccountInfo();
        if (account.getActType() == AccountTypeEnum.BANK) {
            tdSourceAccount.setAccountType(WkAccountTypeEnum.企业账户);
            tdSourceAccount.setAccountNo(account.getActNo());
            tdSourceAccount.setAccountName(account.getActName());
            tdSourceAccount.setChannelNo(WkChannelNoEnum.getByCode(account.getChannelNo()));
            tdSourceAccount.setBankName(account.getActBank());
            tdSourceAccount.setProvince(account.getActProv());
            tdSourceAccount.setCity(account.getActCity());
            tdSourceAccount.setChannelNo(WkChannelNoEnum.getByCode(account.getChannelNo()));
        } else {
            tdSourceAccount.setAccountNo(account.getActNo());
            tdSourceAccount.setAccountType(WkAccountTypeEnum.支付平台账号);
        }
        return tdSourceAccount;
    }

    private void sendNotice(String content) {
        Set<String> receivers = abssqrNoticeManager.getAdminReceivers();
        String bizTitle = "五矿返回结果异常告警";
        String bizCode = "wk";
        abssqrNoticeManager.sendSystemNotice(bizTitle, bizCode, AbssqrNoticeTypeEnum.WkPayManager, content, receivers);

    }
}
