/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.share.pay;

import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalLog;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import com.google.common.collect.Maps;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * 资金单交易完成后的接续处理 工厂类
 * @author zhenxuan.luo
 * @version com.abssqr.plat.core.share.pay: PlanCapitalFinishProcessFactory.java, v 0.1 2019-06-03 21:21 zhenxuan.luo Exp $ 
 */
@Component
public class PlanCapitalFinishProcessFactory {
    
    private final static Logger LOGGER = LoggerFactory.getLogger(PlanCapitalFinishProcessFactory.class);
    
    private final Map<PlanCapitalTransTypeEnum, PlanCapitalFinishProcess> processMap = Maps.newHashMap();
    
    public void registerProcess(PlanCapitalTransTypeEnum transType, PlanCapitalFinishProcess process) {
        processMap.put(transType, process);
    }
    
    public void process(PlanCapitalLog planCapitalLog) {
        VarChecker.checkNotNull(planCapitalLog);
        PlanCapitalFinishProcess process = processMap.get(planCapitalLog.getTransType());
        if (null == process) {
            return;
        }
        
        LogUtil.info(LOGGER, "调用计划[{0}]资金订单[{1}]交易[{2}]后处理器", planCapitalLog.getPlanNo(), planCapitalLog.getSeqNo(),
                planCapitalLog.getTransType());
        
        process.process(planCapitalLog);
    }
}