/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.share.process;

import cn.hutool.json.JSONUtil;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.enums.TransCodeEnum;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.facade.model.trans.CapitalTrader;
import com.abssqr.plat.common.facade.model.trans.ChangeStageParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocHandleParam;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.exception.AbssqrBizException;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.core.service.plan.PlanAcctTitleManager;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.MessageUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * 交易成功 处理计划资金水位
 * @author hanfei
 * @version $Id: PlanCapitalFinishChangeStage.java, v 0.1 2019-07-18 11:55 AM hanfei Exp $
 */
@Service
public class PlanCapitalFinishChangeStage {
    
    private final Logger                    logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private PlanAcctTitleManager planAcctTitleManager;
    @Autowired
    private PlanAllocRepo        planAllocRepo;
    @Autowired
    private PlanAstManager       planAstManager;
    @Autowired
    private PlanRepo             planRepo;
    /**
     * 更新水位金额变化 及登记流水
     * @param capitalOrder
     */
    public void changeAstByCapitalOrder(PlanCapitalOrder capitalOrder){
        Date accountDate = capitalOrder.getAccountDate();
        // 获取 交易码
        PlanCapitalTransTypeEnum transType = capitalOrder.getTransType();
        // 获取交易场景 对应的资金方向 匹配对应的水位变化类型 sub 或 add
        StageChangeTypeEnum changeType = StageChangeTypeEnum.getChangeByFlow(transType.getFlowType());
        LogUtil.info(logger, "计划[{0}]-对手[{1}]-资金单OutBizNo[{2}]-交易码[{3}]-水位变化[{4}]-capitalOrder={5}",
                capitalOrder.getPlanNo(), capitalOrder.getCntPtyCode(),
                capitalOrder.getOutTransNo(), capitalOrder.getTransType().getCode(), changeType.getCode(), capitalOrder);
        
        // 更新水位金额变化 及登记流水
        planAstManager.changePlanAst(accountDate, capitalOrder.getPlanNo(), (planAst, planAstLogList) -> {
            // 分配兑付单独处理
            if (transType.getTransCode() == TransCodeEnum.ALLOC){
                // 1、构建水位变化流水 减少 水位金额 与 减少支付在途金额
                planAstLogList.add(planAstManager.itemChangeStage(accountDate, planAst, TraderTypeEnum.INITIATOR,
                        changeType, capitalOrder.getCfrmAmt(), transType,
                        capitalOrder.getBizNo(), JSONUtil.toJsonStr(capitalOrder)));
                
                // 2、 执行科目入账操作
                this.allocCapitalOrderEntry(capitalOrder);
            }else{
                // 查询对手机构
                PlanBase cntPtyPlan = planRepo.getPlanByNo(capitalOrder.getCntPtyCode());
                // 构建交易双方及支付单上的类型
                CapitalTrader capitalTrader = new CapitalTrader(planAst.getPlanNo(), planAst.getPlanType(),
                        cntPtyPlan.getPlanNo(), cntPtyPlan.getPlanType(), capitalOrder.getTransType());
                // 其他结算支付交易完成处理水位
                // 变化计划资产水位金额与待结算的支出收入金额 登记日志流水
                planAstLogList.addAll(planAstManager.execChangeStageTwoWay(planAst,
                        ChangeStageParam.createParamByCapital(capitalTrader, changeType, capitalOrder)));
            }
        });
    }
    
    
    /**
     * 针对分配兑付单据执行科目入账操作
     * @param capitalOrder
     */
    private void allocCapitalOrderEntry(PlanCapitalOrder capitalOrder) {
        PlanAlloc alloc = planAllocRepo.getPlanAllocDetail(capitalOrder.getBizNo());
        if (alloc == null) {
            throw new AbssqrBizException(MessageUtil.formatMsg("计划分配兑付[{0}]对应的分配计划不存在", capitalOrder.getBizNo()));
        }
        PlanAllocHandleParam handleParam = new PlanAllocHandleParam();
        handleParam.setAlloc(alloc);
        handleParam.setOutTransNo(capitalOrder.getOutTransNo());
        planAcctTitleManager.allocTitleEntry(handleParam, capitalOrder);
    }
}
