/**
 * abssqr.com Inc. 
 * Copyright (c) 2017-2019 All Rights Reserved. 
 */
package com.abssqr.plat.core.share.process;

import cn.hutool.json.JSONUtil;
import com.abssqr.plat.common.facade.enums.PlanCapitalTransTypeEnum;
import com.abssqr.plat.common.facade.enums.StageChangeTypeEnum;
import com.abssqr.plat.common.facade.model.org.OrganizationEntity;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalLog;
import com.abssqr.plat.common.facade.model.trans.PlanClearingOrder;
import com.abssqr.plat.common.model.convertor.PlanClearingOrderConvert;
import com.abssqr.plat.common.model.enums.TraderTypeEnum;
import com.abssqr.plat.common.model.repo.clearing.PlanClearingOrderRepo;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.plan.PlanAstManager;
import com.abssqr.plat.core.service.plan.PlanManager;
import com.abssqr.plat.core.service.plan.calc.PlanHandler;
import com.abssqr.plat.core.service.system.TimeStampUtil;
import com.abssqr.plat.core.share.pay.PlanCapitalFinishProcess;
import com.abssqr.plat.core.share.pay.PlanCapitalFinishProcessFactory;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * 交易成功 购买资产 与支付应付利息 进行回转放逻辑操作
 * @author hanfei
 * @version $Id: PlanCapitalFinishReturnProcess.java, v 0.1 2019-07-18 11:55 AM hanfei Exp $
 */
@Service
public class PlanCapitalFinishReturnProcess implements PlanCapitalFinishProcess {
    
    protected final Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private PlanCapitalFinishProcessFactory planCapitalFinishProcessFactory;
    @Autowired
    private PlanClearingOrderRepo           planClearingOrderRepo;
    @Autowired
    private PlanAstManager                  planAstManager;
    @Autowired
    private PlanManager                     planManager;
    @Autowired
    private OrgAcctDayComponent             orgAcctDayComponent;
    /**
     * 交易成功后进行 补充回转放交易支付单
     * 修改回转放的实现逻辑
         1、交易单成功后变更放款池水位余额
         2.1、增加回转放结算单clearingOrder
         2.2、变更水位记录待结算金额
     * @param capitalLog
     */
    @Override
    public void process(PlanCapitalLog capitalLog) {
        Date accountDate = capitalLog.getAccountDate();
        // 查询放款池计划
        OrganizationEntity organizationEntity = this.getSPVEntity(capitalLog);
        // 获取当前装让系统会计日
        Date trfDate = orgAcctDayComponent.getTrfAcctDay().getStandardDate();
        boolean isNeedClr = PlanHandler.calcNeedClr(organizationEntity, trfDate);
        if (!isNeedClr) {
            LogUtil.info(logger, "放款池计划[{0}]不需要回收转付-OutBizNo[{1}]",
                    organizationEntity.getPlanNo(), capitalLog.getOutBizNo());
            return ;
        }
        // 记录变更增加放款池SPV 计划水位 同时记回转放ClearingOrder
        planAstManager.changePlanAst(accountDate, organizationEntity.getPlanNo(), (spvAst,spvAstLogList) -> {
            /**
             * 1 创建回转放结算单 ClearingOrder
             * ClearingOrder的seqNo为上一笔交易的seqNo关联到上一次的支付交易单据
             * transOrderNo 对应当前交易的bizNo
             */
            PlanClearingOrder clearingOrder = PlanClearingOrderConvert.buildClearingOrder(capitalLog, TimeStampUtil.getCurrentBufferedTimestamp());
            planClearingOrderRepo.createOrder(clearingOrder);
            LogUtil.info(logger, "创建放款池计划回转放结算单：【{0}】", clearingOrder);
            
            // 2 变更 增加 待结算金额 回转放 为资金流出
            spvAstLogList.add(planAstManager.itemChangeStage(clearingOrder.getAccountDate(), spvAst, TraderTypeEnum.INITIATOR,
                    StageChangeTypeEnum.CLEAR, clearingOrder.getTransAmt(), capitalLog.getTransType(),
                    clearingOrder.getSeqNo(), JSONUtil.toJsonStr(clearingOrder)));
        });
    }
    
    @Override
    public void afterPropertiesSet() throws Exception {
        /**
         * 进行回转放
         * 1、转让资产
         * 2、支付应付利息
         * 3、还款冲正 -- 暂时没有
         */
        planCapitalFinishProcessFactory.registerProcess(PlanCapitalTransTypeEnum.BUY_AST, this);
        planCapitalFinishProcessFactory.registerProcess(PlanCapitalTransTypeEnum.PAY_HANG_INTR, this);
        // 暂时没有
        // planCapitalFinishProcessFactory.registerProcess(PlanCapitalTransTypeEnum.REPO_REVERSE, this);
    }
    
    /**
     * 获取放款池计划
     * @param capitalLog
     * @return
     */
    protected OrganizationEntity getSPVEntity(PlanCapitalLog capitalLog) {
        // 查询放款池计划
        OrganizationEntity organizationEntity = planManager.getPlanFullByNo(capitalLog.getCntPtyCode());
        VarChecker.checkNotNull(organizationEntity, "放款池计划[{0}]不存在，不是针对放款计划的打款-OutBizNo[{1}]",
                capitalLog.getCntPtyCode(), capitalLog.getOutBizNo());
        return organizationEntity;
    }
}
