package com.abssqr.plat.test;
/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2018 All Rights Reserved.
 */

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.biz.shared.handler.LoanAstBuyBackMatchHandler;
import com.abssqr.plat.biz.shared.scheduler.job.keeper.node.AstSyncJobKeeper;
import com.abssqr.plat.biz.shared.scheduler.job.keeper.node.BalanceCheckKeeper;
import com.abssqr.plat.biz.shared.scheduler.job.keeper.node.BalanceReportKeeper;
import com.abssqr.plat.biz.shared.scheduler.job.keeper.node.CapitalAstMatchKeeper;
import com.abssqr.plat.biz.shared.scheduler.job.plan.PlanCapitalOrderResJob;
import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.dal.sync.auto.dao.WkAbsMfsBillCtrDAO;
import com.abssqr.plat.common.facade.enums.YNEnum;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.model.domain.accounting.AcctDate;
import com.abssqr.plat.common.model.domain.job.JobControl;
import com.abssqr.plat.common.model.domain.task.ctr.AstSyncTaskCtr;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.acct.AcctDateRepo;
import com.abssqr.plat.common.model.repo.job.JobControlRepository;
import com.abssqr.plat.core.share.pay.PlanCapitalOrderManager;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.SystemDateUtil;
import com.general.system.common.util.VarChecker;
import com.general.system.tool.util.ToolUtil;
import org.apache.commons.lang3.StringUtils;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.mock.mockito.MockitoTestExecutionListener;
import org.springframework.boot.test.mock.mockito.SpyBean;
import org.springframework.test.context.TestExecutionListeners;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

/**
 * 想要使用Mockito，必须添加
 *
 * @author guozhenhua
 * @version com.juhedu.pay.auto.test: com.abssqr.plat.test.BaseAutoTests.java, v 0.1 2018-04-17 14:07 hongwei.wang Exp $
 * @TestExecutionListeners(listeners = MockitoTestExecutionListener.class)'Ø
 */
@TestExecutionListeners(listeners = MockitoTestExecutionListener.class)
public class BaseAutoTests extends BaseAutoWithOutMockTests {

//    @MockBean
//    protected JobControlRepository mockJobControlRepository;
//
//
//    @BeforeMethod
//    protected void stopJob(){
//        JobControl jobControl = new JobControl();
//        jobControl.setIsEff(YNEnum.N);
//        when(mockJobControlRepository.selectByJobName(Mockito.anyString())).thenReturn(jobControl);
//        when(mockJobControlRepository.lockByJobName(Mockito.anyString())).thenReturn(jobControl);
//    }
//
//    protected void mockJobEff(String jobName,YNEnum ynEnum){
//        JobControl jobControl = new JobControl();
//        jobControl.setIsEff(ynEnum);
//        jobControl.setJobName(jobName);
//        when(mockJobControlRepository.selectByJobName(jobName)).thenReturn(jobControl);
//        when(mockJobControlRepository.lockByJobName(jobName)).thenReturn(jobControl);
//    }


    @Autowired
    private PlanCapitalOrderManager planCapitalOrderManager;
    @Autowired
    private PlanCapitalOrderResJob  planCapitalOrderResJob;
    @Autowired
    private WkAbsMfsBillCtrDAO wkAbsMfsBillCtrDAO;
    @Autowired
    private AcctDateDAO acctDateDAO;
    /**
     * 支付单处理及查询结果
     *
     * @param capitalOrders
     */
    protected void capitalOrderMock(List<PlanCapitalOrder> capitalOrders) {
        if (ToolUtil.isEmpty(capitalOrders)) {
            return;
        }

        for (PlanCapitalOrder capitalOrder : capitalOrders) {
            if (capitalOrder == null) {
                continue;
            }
            planCapitalOrderManager.processCapitalOrder(capitalOrder, DateTimeUtil.addMinutes(SystemDateUtil.getSystemDate(), -30));
        }
        // 执行查询
        planCapitalOrderResJob.doExecute();
    }


    @Autowired
    private AcctDateRepo acctDateRepo;

    /**
     * 更新会计日
     *
     * @param strDate
     */
    protected void updateDate(String strDate, String orgCode) {
        Date acctDate = DateUtil.parse(strDate);
        this.updateDate(acctDate, orgCode);
    }
    protected void  updateDate(Date acctDate, String orgCode) {
        OrgCodeEnum org = OrgCodeEnum.getByCode(orgCode);
        VarChecker.checkNotNull(org, "不匹配的机构编码={0}", orgCode);
        AcctDate now = acctDateRepo.getAcctDate(org);
        if (now == null){
            now = new AcctDate();
            now.setOrgCode(org);
            now.setAcctDate(acctDate);
            now.setEff(true);
            acctDateRepo.addAcctDate(now);
        }else if (!DateTimeUtil.isSameDay(now.getAcctDate(), acctDate)) {
            now.setAcctDate(acctDate);
            acctDateRepo.modifyAcctDate(now);
        }
    }

    @Autowired
    private JobControlRepository jobControlRepository;
    
    /**
     * 停止任务
     * @param jobName
     * @return
     */
    protected JobControl invalidJobCtr(String jobName) {
        JobControl jobControl = jobControlRepository.selectByJobName(jobName);
        if (null == jobControl) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.insert(jobControl);
        } else {
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.update(jobControl);
        }

        return jobControl;
    }
    
    /**
     * 开启任务
     * @param jobName
     */
    protected void validJobCtr(String jobName) {
        JobControl jobControl = jobControlRepository.selectByJobName(jobName);
        if (null == jobControl) {
            jobControl = new JobControl();
            jobControl.setJobName(jobName);
            jobControl.setIsEff(YNEnum.N);
            jobControlRepository.insert(jobControl);
        }
        validJobCtr(jobControl);
    }

    protected void validJobCtr(JobControl jobControl) {
        jobControl.setIsEff(YNEnum.Y);
        jobControlRepository.update(jobControl);
    }

    @SpyBean
    private BalanceCheckKeeper balanceCheckKeeper;
    @SpyBean
    private AstSyncJobKeeper   astSyncJobKeeper;

    protected void mockBalanceCheckKeeper(Date date) {
        Mockito.doReturn(date).when(balanceCheckKeeper).getCurrentStanderDate();
    }

    protected void mockAstSyncJobKeeper(Date date) {
        Mockito.doReturn(date).when(astSyncJobKeeper).getCurrentStanderDate();
    }

    @SpyBean
    private CapitalAstMatchKeeper capitalAstMatchKeeper;

    protected void mockCapitalAstMatchKeeper(Date date) {
        Mockito.doReturn(date).when(capitalAstMatchKeeper).getCurrentStanderDate();
    }

    @SpyBean
    private BalanceReportKeeper balanceReportKeeper;
    protected void mockBalanceReportKeeper(Date date) {
        Mockito.doReturn(date).when(balanceReportKeeper).getCurrentStanderDate();
    }

    @SpyBean
    private LoanAstBuyBackMatchHandler loanAstBuyBackMatchHandler;
    protected void mockLoanAstBuyBackMatchHandler(){
        Mockito.doReturn(Boolean.TRUE).when(loanAstBuyBackMatchHandler).canRun();
    }

    protected  void mockCurrentDate(Date date){
        SystemDateUtil systemDateUtil = Mockito.mock(SystemDateUtil.class);
        final Date date1 = Mockito.mock(Date.class);
        Mockito.when(date1.getTime()).thenReturn(date.getTime());
        // Mockito.
        //Mockito.when(systemDateUtil).thenReturn(date);
        //Mockito.doReturn(date).when(systemDateUtil);
    }
    
    
    protected void autoPartition(String orgCode) {
        AstSyncTaskCtr astSyncTaskCtr=new AstSyncTaskCtr();
        astSyncTaskCtr.setOrgCode(orgCode);
        String partitionName = "BILL_CTR_"+astSyncTaskCtr.getOrgCode();
        String pName = wkAbsMfsBillCtrDAO.queryPartitionForCheck(partitionName);
        if(StringUtils.isBlank(pName)){
            wkAbsMfsBillCtrDAO.updateCreatePartition(astSyncTaskCtr.getOrgCode(),partitionName);
        }
    }
}
