package com.abssqr.plat.test.testlink.ast;

import com.abssqr.plat.biz.shared.dispatcher.match.PlanAstMatchDispatcherExecutor;
import com.abssqr.plat.biz.shared.dispatcher.match.PlanAstMatchDispatcherLoader;
import com.abssqr.plat.biz.shared.dispatcher.match.PlanAstMatchDispatcherStarter;
import com.abssqr.plat.biz.shared.dispatcher.planAstStat.PlanAstStatTaskLoader;
import com.abssqr.plat.biz.shared.dispatcher.planAstStat.PlanAstStatTaskStarter;
import com.abssqr.plat.biz.shared.dispatcher.transfer.AstTrTaskExecutor;
import com.abssqr.plat.biz.shared.dispatcher.transfer.AstTrTaskLoader;
import com.abssqr.plat.biz.shared.dispatcher.transfer.AstTrTaskStarter;
import com.abssqr.plat.biz.shared.handler.PlanAstClearingStatHandler;
import com.abssqr.plat.common.facade.enums.CommonTaskTypeEnum;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.abssqr.plat.common.model.wk.domain.sync.WkMfsSyncCtr;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.Assert;
import org.testng.annotations.Test;

import java.util.List;

//@ActiveProfiles("dep")
public class TransferAstNormalTest extends BaseAutoTests {

    private final static Logger LOGGER = LoggerFactory.getLogger(TransferAstNormalTest.class);

    @Autowired
    private PlanAstMatchDispatcherLoader   planAstMatchLoader;
    @Autowired
    private PlanAstMatchDispatcherExecutor planAstMatchExecutor;
    @Autowired
    private PlanAstMatchDispatcherStarter  planAstMatchStarter;
    @Autowired
    private AstTrTaskExecutor              astTrTaskExecutor;
    @Autowired
    private PlanAstStatTaskStarter         planStatTaskStarter;
    @Autowired
    private PlanAstStatTaskLoader          planAstStatTaskLoader;
    @Autowired
    private PlanAstClearingStatHandler     planAstClearingStatHandler;
    @Autowired
    private PlanAstStatTaskRepo            planAstStatTask;
    @Autowired
    private AstTrTaskLoader                astTrTaskLoader;
    @Autowired
    private AstTrTaskStarter               astTrTaskStarter;

    @XTest(relatePath = "testlink/ast/transferAst")
    @Test(dataProvider = "YamlDataProvider", description = "资产转让")
    public void transferAst(ICase iCase, String acctDate, String tfrPack, String trustPlanNo, Boolean flag,
                            WkMfsSyncCtr syncNode) {

        super.invalidJobCtr(astTrTaskStarter.getName());
        super.invalidJobCtr(planStatTaskStarter.getName());
        super.invalidJobCtr(planAstMatchStarter.getName());

        try {
            // 更新会计日期
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
            // 资产筛选
            planAstMatchLoader.setSleepTime(0);
            List<CommonTask> commonTask = planAstMatchLoader.loadToDoNodes(10);
            Assert.assertTrue(commonTask.size() > 0, "没有捞到筛选任务");
            for (CommonTask task : commonTask) {
                if (task.getTaskType().equals(CommonTaskTypeEnum.PACK_MATCH)) {
                    planAstMatchExecutor.execute(task);
                }
            }

            // 资产转让
            List<TrTask> trTasks = astTrTaskLoader.loadToDoNodes(10);
            Assert.assertTrue(trTasks.size() > 0, "没有捞到转让任务");
            for (TrTask trTask : trTasks) {
                astTrTaskExecutor.execute(trTask);
            }
            astTrTaskLoader.loadToDoNodes(10);

            // 清算
            planAstStatTaskLoader.loadToDoNodes(10);
            PlanAstStatTask astStatTask = planAstStatTask.getPlanNo(trustPlanNo);
            if (astStatTask != null) {
                planAstClearingStatHandler.execute(astStatTask, "999912312359");
            }

        }catch (Exception e){
            e.printStackTrace();
            throw  e;
        }finally {
//            super.validJobCtr(astTrTaskStarter.getName());
//            super.validJobCtr(planStatTaskStarter.getName());
//            super.validJobCtr(planAstMatchStarter.getName());
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }
}
