package com.abssqr.plat.test.testlink.plan;

import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackExecutor;
import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackLoader;
import com.abssqr.plat.biz.shared.dispatcher.buyback.LoanAstBuyBackStarter;
import com.abssqr.plat.biz.shared.dispatcher.wkPlanReport.PlanReportTaskExecutor;
import com.abssqr.plat.biz.shared.dispatcher.wkPlanReport.PlanReportTaskLoader;
import com.abssqr.plat.biz.shared.handler.PlanAstClearingOrderHandler;
import com.abssqr.plat.biz.shared.handler.PlanAstClearingStatHandler;
import com.abssqr.plat.biz.shared.handler.PlanAstStatTaskHandler;
import com.abssqr.plat.common.facade.enums.OrgAstStatStatusEnum;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.model.domain.plan.PlanAstStatTask;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.domain.task.TrTask;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.ast.PlanAstStatTaskRepo;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.model.repo.report.PlanReportRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.util.DateTimeUtil;
import com.general.system.common.util.LogUtil;
import com.general.system.tool.util.ToolUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

//@ActiveProfiles("test")
public class PlanBuyBackNormalTest extends BaseAutoTests {
    @Autowired
    private LoanAstBuyBackExecutor loanAstBuyBackExecutor;
    @Autowired
    private LoanAstBuyBackLoader   loanAstBuyBackLoader;
    @Autowired
    private LoanAstBuyBackStarter  loanAstBuyBackStarter;
    
    @Autowired
    private PlanAstStatTaskHandler      planAstStatTaskHandler;
    @Autowired
    private PlanCapitalOrderRepo        planCapitalOrderRepo;
    @Autowired
    private PlanAstClearingStatHandler  clearingStatHandler;
    @Autowired
    private PlanAstStatTaskRepo         planAstStatTask;
    @Autowired
    private PlanAstClearingOrderHandler planAstClearingOrderHandler;
    @Autowired
    private PlanReportRepo              planReportRepo;
    @Autowired
    private PlanReportTaskLoader        planReportTaskLoader;
    @Autowired
    private PlanReportTaskExecutor      planReportTaskExecutor;
    @Autowired
    private PlanRepo                    planRepo;
    @Autowired
    private OrgAcctDayComponent         orgAcctDayComponent;
    
    
    @XTest(relatePath = "testlink/plan/planBuyBack/buyback",
           testOnly = { "plan_buy_back_tr_2002","plan_buy_back_tr_3002","plan_buy_back_tr_4002","plan_buy_back_tr_5002" })
    @Test(dataProvider = "YamlDataProvider", description = "资产回购")
    public void execute(ICase iCase, String acctDate, String planNo, String spvNo) throws ParseException {
        try {
            // 更新会计日期
            DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
            Date strDate = format.parse(acctDate);
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
    
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            
            this.invalidJobCtr(loanAstBuyBackStarter.getName());
            List<TrTask> trTasks = loanAstBuyBackLoader.loadToDoNodes(1);
            loanAstBuyBackExecutor.execute(trTasks.get(0));
            // ctr_task 更新succ
            loanAstBuyBackLoader.loadToDoNodes(1);
            
            // 清算
            // 计划资产状态汇总
            LogUtil.info(logger, " ---- 7 清算汇总");
            planAstStatTaskHandler.handle(planNo, trfDate);
            planAstStatTaskHandler.handle(spvNo, trfDate);
            
            PlanAstStatTask astStatTask1 = planAstStatTask.getPlanNo(planNo);
            if (astStatTask1 != null) {
                astStatTask1.setStatus(OrgAstStatStatusEnum.PROCESS);
                planAstStatTask.updateTask(astStatTask1);
                clearingStatHandler.execute(astStatTask1, "999912312359");
            }
            PlanAstStatTask astStatTask2 = planAstStatTask.getPlanNo(spvNo);
            if (astStatTask2 != null) {
                clearingStatHandler.execute(astStatTask2, "999912312359");
                astStatTask2.setAccountDate(strDate);
                astStatTask2.setMinId(null);
                astStatTask2.setStatus(OrgAstStatStatusEnum.PROCESS);
                planAstStatTask.updateTask(astStatTask2);
                clearingStatHandler.execute(astStatTask2, "999912312359");
            }
            
            // 结算任务处理
            PlanBase plan = planRepo.getPlanByNo(planNo);
            PlanBase spv = planRepo.getPlanByNo(spvNo);
            LogUtil.info(logger, " ---- 8 结算处理");
            planAstClearingOrderHandler.handle(plan);
            planAstClearingOrderHandler.handle(spv);
            
            // wk交易调用
            // 资金订单状态由wait-->succ， 记得修改next_exec_time <= sysdate
            LogUtil.info(logger, " ---- 9 支付处理");
            List<PlanCapitalOrder> capitalOrders = planCapitalOrderRepo.getInitListByPlanNo(planNo);
            capitalOrders.addAll(planCapitalOrderRepo.getInitListByPlanNo(spvNo));
            super.capitalOrderMock(capitalOrders);
            
            // 放款池可能会有回转放的结算单需要执行
            planAstClearingOrderHandler.handle(spv);
            List<PlanCapitalOrder> spvReturn = planCapitalOrderRepo.getInitListByPlanNo(spv.getPlanNo());
            super.capitalOrderMock(spvReturn);
            
            
            // Item_default 初始化
            planReportRepo.defaultRPT001Init(spv.getPlanNo(), strDate);
            planReportRepo.defaultRPT001Init(plan.getPlanNo(), strDate);
    
            LogUtil.info(logger, " ---- 10 回购后生成 计划报表");
            List<CommonTask> tasks = planReportTaskLoader.loadToDoNodes(20);
            while (tasks.size() > 0) {
                for (CommonTask task : tasks) {
                    if (ToolUtil.equals(task.getContext(), planNo))
                        planReportTaskExecutor.doExecute(task);
                }
                
                for (CommonTask task : tasks) {
                    if (ToolUtil.equals(task.getContext(), spvNo))
                        planReportTaskExecutor.doExecute(task);
                }
                
                tasks = planReportTaskLoader.loadToDoNodes(20);
            }
            super.updateDate(DateTimeUtil.addDays(strDate, 1), OrgCodeEnum.ABS.getCode());
            tasks = planReportTaskLoader.loadToDoNodes(20);
            while (tasks.size() > 0) {
                for (CommonTask task : tasks) {
                    if (ToolUtil.equals(task.getContext(), planNo))
                        planReportTaskExecutor.doExecute(task);
                }
        
                for (CommonTask task : tasks) {
                    if (ToolUtil.equals(task.getContext(), spvNo))
                        planReportTaskExecutor.doExecute(task);
                }
        
                tasks = planReportTaskLoader.loadToDoNodes(20);
            }
        } catch (Exception e) {
            throw e;
        } finally {
            // 校验数据
            super.checkDB(iCase);
            //            super.clearDB(iCase);
        }
        
    }
}
