/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */

package com.abssqr.plat.test.unittest.plan;

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.biz.shared.handler.PlanFundHandler;
import com.abssqr.plat.common.dal.mysql.auto.dao.BizDataChangeDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.BizDataChangeDO;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.bpm.domain.result.ProcessAfterResult;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.general.system.core.common.constant.factory.ConstantFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.util.Comparator;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 追加投资结构
 * @author xiachenxiang
 * @version com.abssqr.plat.test.unittest.plan: PlanAppendStructure.java, v 0.1 2019-11-13 10:29 AM xiachenxiang Exp $ 
 */
public class PlanAppendStructureTest extends BaseAutoTests {

    @Autowired
    private PlanService         planService;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;
    @Autowired
    private BizDataChangeDAO    bizDataChangeDAO;
    @Autowired
    private ChangeManager       changeManager;
    @Autowired
    private PlanRepo            planRepo;
    @Autowired
    private PlanFundHandler     planFundHandler;
    @Autowired
    private PlanAllocRepo       planAllocRepo;


    @XTest(relatePath = "unittest/plan/planAppendTest/planAdd")
    @Test(dataProvider = "YamlDataProvider", description = "计划新增", priority = 1)
    public void create(ICase iCase, String acctDate, PlanOprParam planParam, List<LinkedHashMap> allocCheckList, List<LinkedHashMap> itemCheckList) {

        ConstantFactory.me().securityLogin("administrator");

        try {
            
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
    
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            planService.edit(planParam);

            BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(planParam.getPlanNo(), "PENDING");
            if (bizData != null) {
                // 审批-->创建水位与科目
                ProcessAfterResult result = new ProcessAfterResult();
                result.setBusinessKey(bizData.getChangeNo());
                result.setStatusEnum(ApproveStatusEnum.PASSED);
                changeManager.handle(result);
            }

            // 执行余额查询
            PlanBase planEntity = planRepo.getPlanByNo(planParam.getPlanNo());
            planFundHandler.executeQueryBalance(planEntity, trfDate);
            List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(planParam.getPlanNo());

            List<Date> clearDatelist = planAllocs.stream().map(PlanAlloc::getClearDate).collect(Collectors.toList());
            List<Date> checkDatelist = allocCheckList.stream().map(item -> {
                return DateUtil.parse(String.valueOf(item.get("payDate")));
            }).collect(Collectors.toList());
            // 校验分配计划
            VarChecker.checkEquals(clearDatelist, checkDatelist);
            // 校验分配计划数量
            planAllocs.sort(Comparator.comparing(PlanAlloc::getClearDate));
            List<Integer> allocItemList = planAllocs.stream().map(item -> {
                return item.getPlanAllocItemList().size();
            }).collect(Collectors.toList());
            List<Integer> allocItemCheckList = itemCheckList.stream().map(item -> {
                return (Integer) item.get("cnt");
            }).collect(Collectors.toList());
            VarChecker.checkEquals(allocItemList, allocItemCheckList);
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }

    @XTest(relatePath = "unittest/plan/planAppendTest/planAppend")
    @Test(dataProvider = "YamlDataProvider", description = "追加投资结构", priority = 2)
    public void append(ICase iCase, String acctDate, PlanOprParam planParam, List<LinkedHashMap> allocCheckList, List<LinkedHashMap> itemCheckList) {

        ConstantFactory.me().securityLogin("administrator");

        // 更新会计日期
        try {
    
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            
            planService.edit(planParam);

            BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(planParam.getPlanNo(), "PENDING");
            if (bizData != null) {
                // 审批-->创建水位与科目
                ProcessAfterResult result = new ProcessAfterResult();
                result.setBusinessKey(bizData.getChangeNo());
                result.setStatusEnum(ApproveStatusEnum.PASSED);
                changeManager.handle(result);
            }


            // 执行余额查询
            PlanBase planEntity = planRepo.getPlanByNo(planParam.getPlanNo());
            planFundHandler.executeQueryBalance(planEntity, trfDate);
            List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(planParam.getPlanNo());

            List<Date> clearDatelist = planAllocs.stream().map(PlanAlloc::getClearDate).collect(Collectors.toList());
            List<Date> checkDatelist = allocCheckList.stream().map(item -> {
                return DateUtil.parse((String) item.get("payDate"));
            }).collect(Collectors.toList());
            // 校验分配计划
            VarChecker.checkEquals(clearDatelist, checkDatelist);
            // 校验分配计划数量
            planAllocs.sort(Comparator.comparing(PlanAlloc::getClearDate));
            List<Integer> allocItemList = planAllocs.stream().map(item -> {
                return item.getPlanAllocItemList().size();
            }).collect(Collectors.toList());
            List<Integer> allocItemCheckList = itemCheckList.stream().map(item -> {
                return (Integer) item.get("cnt");
            }).collect(Collectors.toList());
            VarChecker.checkEquals(allocItemList, allocItemCheckList);
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }
}
