/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.test.unittest.plan;

import cn.hutool.core.date.DateUtil;
import com.abssqr.plat.biz.shared.handler.PlanFundHandler;
import com.abssqr.plat.common.dal.mysql.auto.dao.AcctDateDAO;
import com.abssqr.plat.common.dal.mysql.auto.dao.BizDataChangeDAO;
import com.abssqr.plat.common.dal.mysql.auto.dataobject.BizDataChangeDO;
import com.abssqr.plat.common.facade.model.plan.PlanAlloc;
import com.abssqr.plat.common.facade.model.plan.PlanBase;
import com.abssqr.plat.common.facade.param.plan.PlanOprParam;
import com.abssqr.plat.common.facade.service.plan.PlanService;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanAllocRepo;
import com.abssqr.plat.common.model.repo.plan.PlanRepo;
import com.abssqr.plat.common.util.base.Day;
import com.abssqr.plat.core.service.accounting.OrgAcctDayComponent;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.bpm.domain.result.ProcessAfterResult;
import com.general.system.bpm.enums.ApproveStatusEnum;
import com.general.system.common.util.VarChecker;
import com.general.system.core.common.constant.factory.ConstantFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.util.Comparator;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author yangcheng
 * @version PlanEditUnitTest.java, v0.12019-06-1016:26 yangchengExp$
 */
public class PlanEditUnitTest extends BaseAutoTests {
    @Autowired
    private PlanService         planService;
    @Autowired
    private AcctDateDAO         acctDateDAO;
    @Autowired
    private BizDataChangeDAO    bizDataChangeDAO;
    @Autowired
    private ChangeManager       changeManager;
    @Autowired
    private PlanRepo            planRepo;
    @Autowired
    private PlanFundHandler     planFundHandler;
    @Autowired
    private PlanAllocRepo       planAllocRepo;
    @Autowired
    private OrgAcctDayComponent orgAcctDayComponent;

    @XTest(relatePath = "unittest/plan/planTest")
    @Test(dataProvider = "YamlDataProvider", description = "计划新增")
    public void create(ICase iCase, String acctDate,  PlanOprParam param, List<LinkedHashMap> allocCheckList, List<LinkedHashMap> itemCheckList) {

        ConstantFactory.me().securityLogin("administrator");

        // 更新会计日期
        try {
            super.updateDate(acctDate, OrgCodeEnum.ABS.getCode());
            Day trfDate = orgAcctDayComponent.getTrfAcctDay();
            
            planService.edit(param);

            BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(param.getPlanNo(), "PENDING");
            if (bizData != null) {
                // 审批-->创建水位与科目
                ProcessAfterResult result = new ProcessAfterResult();
                result.setBusinessKey(bizData.getChangeNo());
                result.setStatusEnum(ApproveStatusEnum.PASSED);
                changeManager.handle(result);

                // 执行余额查询
                PlanBase planEntity = planRepo.getPlanByNo(param.getPlanNo());
                planFundHandler.executeQueryBalance(planEntity, trfDate);
                List<PlanAlloc> planAllocs = planAllocRepo.queryByPlanNo(param.getPlanNo());

                List<Date> clearDatelist = planAllocs.stream().map(PlanAlloc::getClearDate).collect(Collectors.toList());
                List<Date> checkDatelist = allocCheckList.stream().map(item -> {
                    return DateUtil.parse((String) item.get("payDate"));
                }).collect(Collectors.toList());
                // 校验分配计划
                VarChecker.checkEquals(clearDatelist, checkDatelist);
                // 校验分配计划数量
                planAllocs.sort(Comparator.comparing(PlanAlloc::getClearDate));
                List<Integer> allocItemList = planAllocs.stream().map(item -> {
                    return item.getPlanAllocItemList().size();
                }).collect(Collectors.toList());
                List<Integer> allocItemCheckList = itemCheckList.stream().map(item -> {
                    return (Integer) item.get("cnt");
                }).collect(Collectors.toList());
                VarChecker.checkEquals(allocItemList, allocItemCheckList);
            }
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }

    }
}
