/**
 * abssqr.comInc.
 * Copyright(c)2019-2019AllRightsReserved.
 */
package com.abssqr.plat.test.unittest.planAlloc;

import com.abssqr.plat.biz.shared.scheduler.job.plan.PlanDailyJob;
import com.abssqr.plat.common.dal.mysql.auto.dao.BizDataChangeDAO;
import com.abssqr.plat.common.facade.model.plan.PlanCapitalOrder;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocDetailQryParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocPayParam;
import com.abssqr.plat.common.facade.param.plan.alloc.PlanAllocQryParam;
import com.abssqr.plat.common.facade.result.plan.PlanAllocPayInfoVO;
import com.abssqr.plat.common.facade.result.plan.PlanAllocVO;
import com.abssqr.plat.common.facade.service.plan.PlanAllocPayService;
import com.abssqr.plat.common.facade.service.plan.PlanAllocService;
import com.abssqr.plat.common.model.enums.OrgCodeEnum;
import com.abssqr.plat.common.model.repo.plan.PlanCapitalOrderRepo;
import com.abssqr.plat.core.service.change.ChangeManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

import java.util.List;

import static org.unitils.reflectionassert.ReflectionAssert.assertReflectionEquals;

/**
 * 分配兑付测试
 * @author hanfei
 * @version PlanAllocUnitTest.java, v0.1 2020-01-09 20:21 hanfei
 */
//@ActiveProfiles("dep")
public class PlanAllocUnitTest extends BaseAutoTests {
    
    @Autowired
    private PlanAllocService     planAllocService;
    @Autowired
    private PlanAllocPayService  planAllocPayService;
    @Autowired
    private PlanDailyJob         planDailyJob;
    @Autowired
    private BizDataChangeDAO     bizDataChangeDAO;
    @Autowired
    private ChangeManager        changeManager;
    @Autowired
    private PlanCapitalOrderRepo planCapitalOrderRepo;
    
    @XTest(relatePath = "unittest/PlanAlloc/query")
    @Test(dataProvider = "YamlDataProvider", description = "查询分配计划")
    public void getPlanAllocTest(ICase iCase, PlanAllocQryParam param){
        try{
            planAllocService.getPlanAlloc(param);
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
    
    /**
     * 1、新增分配计划
     * 2、保存分配计划
     * 3、提交分配计划、并审批通过、并每日任务冻结
     */
//    @XTest(relatePath = "unittest/PlanAlloc/init")
//    @Test(dataProvider = "YamlDataProvider", description = "初始化分配计划,日初冻结")
//    public void initPlanAllocFrozenTest(ICase iCase, String strDate, PlanAllocEditParam param, String planType,
//                                    boolean submit, boolean approve, boolean dailyJob){
//        try{
//            PlanTypeEnum typeEnum = PlanTypeEnum.getByCode(planType);
//            PlanAllocVO allocVO = planAllocService.edit(param, typeEnum, submit);
//
//            // 审批
//            if (approve){
//                ConstantFactory.me().securityLogin("administrator");
//                BizDataChangeDO bizData = bizDataChangeDAO.getByBizNoAndStatus(allocVO.getAllocNo(), "PENDING");
//                if (bizData != null) {
//                    // 审批通过
//                    ProcessAfterResult result = new ProcessAfterResult();
//                    result.setBusinessKey(bizData.getChangeNo());
//                    result.setStatusEnum(ApproveStatusEnum.PASSED);
//                    changeManager.handle(result);
//                }
//            }
//            // 每日任务
//            if (dailyJob){
//                super.updateDate(strDate);
//                planDailyJob.doExecute();
//            }
//        } finally {
//            // 校验数据
//            super.checkDB(iCase);
//            super.clearDB(iCase);
//        }
//    }
    
    @XTest(relatePath = "unittest/PlanAlloc/info")
    @Test(dataProvider = "YamlDataProvider", description = "确认分配详情")
    public void detailPlanAlloc(ICase iCase, PlanAllocPayParam param, PlanAllocPayInfoVO infoVOCheck){
        try{
            PlanAllocPayInfoVO infoVO = planAllocPayService.allocInfo(param);
            // 校验数据
            assertReflectionEquals(infoVO, infoVOCheck);
        } finally {
            super.clearDB(iCase);
        }
    }
    
    /**
     * 1、放款池线上分配
     * 2、放款池线下分配
     * 3、承接池线上分配
     * 4、承接池线下分配
     */
    @XTest(relatePath = "unittest/PlanAlloc/pay")
    @Test(dataProvider = "YamlDataProvider", description = "分配计划分配")
    public void payPlanAllocTest(ICase iCase, String strDate, PlanAllocPayParam param, Boolean pay){
        try{
            super.updateDate(strDate, OrgCodeEnum.ABS.getCode());
            
            planAllocPayService.allocPay(param);
            
            if (pay) {
                PlanAllocDetailQryParam qryParam = new PlanAllocDetailQryParam();
                qryParam.setAllocNo(param.getAllocNo());
                PlanAllocVO vo = planAllocService.getPlanAllocDetail(qryParam);
                
                List<PlanCapitalOrder> capitalOrders = planCapitalOrderRepo.getInitListByPlanNo(vo.getPlanNo());
                super.capitalOrderMock(capitalOrders);
            }
        } finally {
            // 校验数据
            super.checkDB(iCase);
            super.clearDB(iCase);
        }
    }
}
