/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2019 All Rights Reserved.
 */
package com.abssqr.plat.test.unittest.wk;

import cn.hutool.core.collection.CollectionUtil;
import com.abssqr.plat.common.facade.enums.AccountAttributeEnum;
import com.abssqr.plat.common.facade.model.account.Account;
import com.abssqr.plat.common.integration.wk.domain.opr.request.acct.WkAccount;
import com.abssqr.plat.common.integration.wk.domain.qry.response.trade.TradeResult;
import com.abssqr.plat.common.integration.wk.enums.WkChannelNoEnum;
import com.abssqr.plat.common.util.base.autoconfig.AbsConfig;
import com.abssqr.plat.core.service.wk.pay.WkPayManager;
import com.abssqr.plat.test.BaseAutoTests;
import com.abssqr.plat.test.BaseAutoWithOutMockTests;
import com.abssqr.test.annotation.XTest;
import com.abssqr.test.icase.ICase;
import com.general.system.common.model.Money;
import com.general.system.common.model.exception.BaseCommonException;
import com.general.system.common.util.LogUtil;
import com.general.system.common.util.VarChecker;
import org.springframework.beans.factory.annotation.Autowired;
import org.testng.annotations.Test;

/**
 * 五矿接口测试
 * @author hanfei
 * @version $Id: WkPayManagerTest.java, v 0.1 2019-06-27 2:35 PM hanfei Exp $
 */
public class WkPayManagerTest extends BaseAutoWithOutMockTests {

    @Autowired
    private WkPayManager wkPayManager;

    public final static String testBag = "manager/pay";
    @Autowired
    AbsConfig absConfig;
    /**
     * 租户签约与维护
     *
     * @return
     */
    @XTest(relatePath = testBag + "/project")
    @Test(dataProvider = "YamlDataProvider", description = "租户签约与维护")
    public void project(ICase iCase) {
        boolean b = wkPayManager.project();
        VarChecker.checkEquals(b, true, "租户签约与维护-异常");
    }

    /**
     * 账户信息维护
     *
     * 账户 1 ：账号  755915676910701  账户名称： 企业网银新20161058    开户分行：招商银行深圳分行
     * 账户 2 ：账号  755915676910806  账户名称： 企业网银新20161058    开户分行：招商银行深圳分行
     *
     * 账户 3 ：账号  755915712310502  账户名称： 企业网银新20161397    开户分行：招商银行深圳分行
     * 账户 4 ：账号  755915712310607  账户名称： 企业网银新20161397    开户分行：招商银行深圳分行
     * 000004_001_P190625   ===  755915676910701
     * 000004_001_P830330   ===  755915676910806
     *
     *
     * @param planNo
     * @param planName
     * @return
     */
    @XTest(relatePath = testBag + "/account")
    @Test(dataProvider = "YamlDataProvider", description = "账户信息维护")
    public void account(ICase iCase, String planNo, String planName) {
        WkAccount account = wkPayManager.createAccount(planNo, planName, AccountAttributeEnum.NORMAL);
        LogUtil.info(logger, account.toString());
        VarChecker.checkNotNull(account, "账户信息维护-异常");
    }

    /**
     * 账户余额查询 B100002
     * 最多50条
     *
     * @param accountNo
     * @return
     */
    @XTest(relatePath = testBag + "/balance")
    @Test(dataProvider = "YamlDataProvider", description = "账户余额查询")
    public void balance(ICase iCase, String accountNo) {
        try {
            Money amt = wkPayManager.balance(accountNo, WkChannelNoEnum.BC_YQ0001);
        }catch (BaseCommonException e){
            e.printStackTrace();
        }
        checkDB(iCase);
    }

    /**
     * 代理交易
     *
     * @return
     */
    @XTest(relatePath = testBag + "/trade")
    @Test(dataProvider = "YamlDataProvider", description = "代理交易")
    public void trade(ICase iCase, Long money, Account sourceAcct, Account targetAcct) {
        String enNo = wkPayManager.trade(sourceAcct, targetAcct, Money.createWithCent(money),"钉钉");
        LogUtil.info(logger, enNo);

        // 休息一下
        try {
            Thread.sleep(3000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        // 代理交易结果查询
        TradeResult tradeResult = wkPayManager.tradeRstQry(CollectionUtil.newArrayList(enNo));
        LogUtil.info(logger, tradeResult.toString());
    }
  /**
     * 代理交易
     *
     * @return
     */
    @XTest(relatePath = testBag + "/trade")
    @Test(dataProvider = "YamlDataProvider", description = "代理交易")
    public void trade2(ICase iCase, Long money, Account sourceAcct, Account targetAcct) {
        String enNo = wkPayManager.trade(sourceAcct, targetAcct, Money.createWithCent(money),null);
        LogUtil.info(logger, enNo);
    }

    /**
     * 代理交易结果查询
     *
     * @param tradeNo
     * @return
     */
    @XTest(relatePath = testBag + "/tradeRstQry")
    @Test(dataProvider = "YamlDataProvider", description = "代理交易结果查询")
    public void tradeRstQry(ICase iCase, String tradeNo) {
        TradeResult tradeResult = wkPayManager.tradeRstQry(CollectionUtil.newArrayList(tradeNo+1));
        LogUtil.info(logger, tradeResult.toString());
    }
}
